#
# -*- coding: utf-8 -*-
# Copyright 2023 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

"""
The parse_xml plugin code
"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type

import os
import re

from ansible.errors import AnsibleFilterError
from ansible.module_utils._text import to_native
from ansible.module_utils.common._collections_compat import Mapping
from ansible.module_utils.six import iteritems, string_types

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import Template


try:
    import yaml

    HAS_YAML = True
except ImportError:
    HAS_YAML = False


def _raise_error(msg):
    raise AnsibleFilterError(msg)


def re_matchall(regex, value):
    objects = list()
    for match in re.findall(regex.pattern, value, re.M):
        obj = {}
        if regex.groupindex:
            for name, index in iteritems(regex.groupindex):
                if len(regex.groupindex) == 1:
                    obj[name] = match
                else:
                    obj[name] = match[index - 1]
            objects.append(obj)
    return objects


def re_search(regex, value):
    obj = {}
    match = regex.search(value, re.M)
    if match:
        items = list(match.groups())
        if regex.groupindex:
            for name, index in iteritems(regex.groupindex):
                obj[name] = items[index - 1]
    return obj


def re_finditer(regex, value):
    iter_obj = re.finditer(regex, value)
    values = None
    for each in iter_obj:
        if not values:
            values = each.groupdict()
        else:
            # for backward compatibility
            values.update(each.groupdict())
        # for backward compatibility
        values["match"] = list(each.groups())
        groups = each.groupdict()
        for group in groups:
            if not values.get("match_all"):
                values["match_all"] = dict()
            if not values["match_all"].get(group):
                values["match_all"][group] = list()
            values["match_all"][group].append(groups[group])
    return values


def parse_cli(output, tmpl):
    if not isinstance(output, string_types):
        _raise_error(
            "parse_cli input should be a string, but was given a input of %s" % (type(output))
        )

    if not os.path.exists(tmpl):
        _raise_error("unable to locate parse_cli template: %s" % tmpl)

    try:
        template = Template()
    except ImportError as exc:
        _raise_error(to_native(exc))

    with open(tmpl) as tmpl_fh:
        tmpl_content = tmpl_fh.read()

    spec = yaml.safe_load(tmpl_content)
    obj = {}

    for name, attrs in iteritems(spec["keys"]):
        value = attrs["value"]

        try:
            variables = spec.get("vars", {})
            value = template(value, variables)
        except Exception:
            pass

        if "start_block" in attrs and "end_block" in attrs:
            start_block = re.compile(attrs["start_block"])
            end_block = re.compile(attrs["end_block"])

            blocks = list()
            lines = None
            block_started = False

            for line in output.split("\n"):
                match_start = start_block.match(line)
                match_end = end_block.match(line)

                if match_start:
                    lines = list()
                    lines.append(line)
                    block_started = True

                elif match_end:
                    if lines:
                        lines.append(line)
                        blocks.append("\n".join(lines))
                        lines = None
                    block_started = False

                elif block_started:
                    if lines:
                        lines.append(line)

            regex_items = [re.compile(r) for r in attrs["items"]]
            objects = list()

            for block in blocks:
                if isinstance(value, Mapping) and "key" not in value:
                    items = list()
                    for regex in regex_items:
                        items.append(re_finditer(regex, block))

                    obj = {}
                    for k, v in iteritems(value):
                        try:
                            obj[k] = template(v, {"item": items}, fail_on_undefined=False)
                        except Exception:
                            obj[k] = None
                    objects.append(obj)

                elif isinstance(value, Mapping):
                    items = list()
                    for regex in regex_items:
                        items.append(re_finditer(regex, block))

                    key = template(value["key"], {"item": items})
                    values = dict(
                        [(k, template(v, {"item": items})) for k, v in iteritems(value["values"])]
                    )
                    objects.append({key: values})

            return objects

        elif "items" in attrs:
            regexp = re.compile(attrs["items"])
            when = attrs.get("when")
            conditional = "{%% if %s %%}True{%% else %%}False{%% endif %%}" % when

            if isinstance(value, Mapping) and "key" not in value:
                values = list()

                for item in re_matchall(regexp, output):
                    entry = {}

                    for item_key, item_value in iteritems(value):
                        entry[item_key] = template(item_value, {"item": item})

                    if when:
                        if template(conditional, {"item": entry}):
                            values.append(entry)
                    else:
                        values.append(entry)

                obj[name] = values

            elif isinstance(value, Mapping):
                values = dict()

                for item in re_matchall(regexp, output):
                    entry = {}

                    for item_key, item_value in iteritems(value["values"]):
                        entry[item_key] = template(item_value, {"item": item})

                    key = template(value["key"], {"item": item})

                    if when:
                        if template(conditional, {"item": {"key": key, "value": entry}}):
                            values[key] = entry
                    else:
                        values[key] = entry

                obj[name] = values

            else:
                item = re_search(regexp, output)
                obj[name] = template(value, {"item": item})

        else:
            obj[name] = value

    return obj



# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The eos_l2_interfaces class
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to it's desired end-state is
created
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.cfg.base import (
    ConfigBase,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    param_list_to_dict,
    to_list,
)

from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.facts import Facts
from ansible_collections.arista.eos.plugins.module_utils.network.eos.utils.utils import (
    normalize_interface,
    vlan_range_to_list,
)


class L2_interfaces(ConfigBase):
    """
    The eos_l2_interfaces class
    """

    gather_subset = ["!all", "!min"]

    gather_network_resources = ["l2_interfaces"]

    def get_l2_interfaces_facts(self, data=None):
        """Get the 'facts' (the current configuration)

        :rtype: A dictionary
        :returns: The current configuration as a dictionary
        """
        facts, _warnings = Facts(self._module).get_facts(
            self.gather_subset,
            self.gather_network_resources,
            data=data,
        )
        l2_interfaces_facts = facts["ansible_network_resources"].get(
            "l2_interfaces",
        )
        if not l2_interfaces_facts:
            return []
        return l2_interfaces_facts

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        result = {"changed": False}
        commands = list()
        warnings = list()

        if self.state in self.ACTION_STATES:
            existing_l2_interfaces_facts = self.get_l2_interfaces_facts()
        else:
            existing_l2_interfaces_facts = []

        if self.state in self.ACTION_STATES or self.state == "rendered":
            commands.extend(self.set_config(existing_l2_interfaces_facts))

        if commands and self.state in self.ACTION_STATES:
            if not self._module.check_mode:
                self._connection.edit_config(commands)
            result["changed"] = True

        if self.state in self.ACTION_STATES:
            result["commands"] = commands

        if self.state in self.ACTION_STATES or self.state == "gathered":
            changed_l2_interfaces_facts = self.get_l2_interfaces_facts()

        elif self.state == "rendered":
            result["rendered"] = commands

        elif self.state == "parsed":
            running_config = self._module.params["running_config"]
            if not running_config:
                self._module.fail_json(
                    msg="value of running_config parameter must not be empty for state parsed",
                )
            result["parsed"] = self.get_l2_interfaces_facts(
                data=running_config,
            )

        if self.state in self.ACTION_STATES:
            result["before"] = existing_l2_interfaces_facts
            if result["changed"]:
                result["after"] = changed_l2_interfaces_facts

        elif self.state == "gathered":
            result["gathered"] = changed_l2_interfaces_facts

        result["warnings"] = warnings
        return result

    def set_config(self, existing_l2_interfaces_facts):
        """Collect the configuration from the args passed to the module,
            collect the current configuration (as a dict from facts)

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        want = self._module.params["config"]
        have = existing_l2_interfaces_facts
        resp = self.set_state(want, have)
        return to_list(resp)

    def set_state(self, want, have):
        """Select the appropriate function based on the state provided

        :param want: the desired configuration as a dictionary
        :param have: the current configuration as a dictionary
        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        state = self._module.params["state"]
        if state in ("merged", "replaced", "overridden", "rendered") and not want:
            self._module.fail_json(
                msg="value of config parameter must not be empty for state {0}".format(
                    state,
                ),
            )
        want = param_list_to_dict(want)
        have = param_list_to_dict(have)
        commands = []
        if state == "overridden":
            commands = self._state_overridden(want, have)
        elif state == "deleted":
            commands = self._state_deleted(want, have)
        elif state == "merged" or state == "rendered":
            commands = self._state_merged(want, have)
        elif state == "replaced":
            commands = self._state_replaced(want, have)
        return commands

    @staticmethod
    def _state_replaced(want, have):
        """The command generator when state is replaced

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        commands = []
        for key, desired in want.items():
            interface_name = normalize_interface(key)
            if interface_name in have:
                extant = have[interface_name]
            else:
                extant = dict()

            intf_commands = set_interface(desired, extant)
            intf_commands.extend(clear_interface(desired, extant))

            if intf_commands:
                commands.append("interface {0}".format(interface_name))
                commands.extend(intf_commands)

        return commands

    @staticmethod
    def _state_overridden(want, have):
        """The command generator when state is overridden

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        commands = []
        for key, extant in have.items():
            if key in want:
                desired = want[key]
            else:
                desired = dict()

            intf_commands = set_interface(desired, extant)
            intf_commands.extend(clear_interface(desired, extant))

            if intf_commands:
                commands.append("interface {0}".format(key))
                commands.extend(intf_commands)

        return commands

    @staticmethod
    def _state_merged(want, have):
        """The command generator when state is merged

        :rtype: A list
        :returns: the commands necessary to merge the provided into
                  the current configuration
        """
        commands = []
        for key, desired in want.items():
            interface_name = normalize_interface(key)
            if interface_name in have:
                extant = have[interface_name]
            else:
                extant = dict()

            intf_commands = set_interface(desired, extant)

            if intf_commands:
                commands.append("interface {0}".format(interface_name))
                commands.extend(intf_commands)

        return commands

    @staticmethod
    def _state_deleted(want, have):
        """The command generator when state is deleted

        :rtype: A list
        :returns: the commands necessary to remove the current configuration
                  of the provided objects
        """
        commands = []
        for key in want:
            desired = dict()
            if key in have:
                extant = have[key]
            else:
                continue
            intf_commands = clear_interface(desired, extant)

            if intf_commands:
                commands.append("interface {0}".format(key))
                commands.extend(intf_commands)

        return commands


def set_interface(want, have):
    commands = []

    want_mode = want.get("mode")
    if want_mode and want_mode != have.get("mode"):
        commands.append("switchport mode {0}".format(want_mode))

    wants_access = want.get("access")
    if wants_access:
        access_vlan = wants_access.get("vlan")
        if access_vlan and access_vlan != have.get("access", {}).get("vlan"):
            commands.append("switchport access vlan {0}".format(access_vlan))

    wants_trunk = want.get("trunk")
    if wants_trunk:
        allowed_vlans = []
        has_allowed_vlans = {}
        want_allowed_vlans = {}
        has_trunk = have.get("trunk", {})
        native_vlan = wants_trunk.get("native_vlan")
        if native_vlan and native_vlan != has_trunk.get("native_vlan"):
            commands.append(
                "switchport trunk native vlan {0}".format(native_vlan),
            )
        for con in [want, have]:
            expand_trunk_allowed_vlans(con)
        want_allowed_vlans = want["trunk"].get("trunk_allowed_vlans")
        if has_trunk:
            has_allowed_vlans = has_trunk.get("trunk_allowed_vlans")

        if want_allowed_vlans and has_allowed_vlans:
            allowed_vlans = list(
                set(want_allowed_vlans.split(",")) - set(has_allowed_vlans.split(",")),
            )
        elif want_allowed_vlans:
            allowed_vlans = want_allowed_vlans.split(",")
        if allowed_vlans:
            allowed_vlans.sort()
            allowed_vlans = ",".join(
                ["{0}".format(vlan) for vlan in allowed_vlans],
            )
            if has_allowed_vlans:
                commands.append(
                    "switchport trunk allowed vlan add {0}".format(
                        allowed_vlans,
                    ),
                )
            else:
                commands.append(
                    "switchport trunk allowed vlan {0}".format(allowed_vlans),
                )
    return commands


def expand_trunk_allowed_vlans(want):
    if not want:
        return None
    if want.get("trunk"):
        if "trunk_allowed_vlans" in want["trunk"]:
            allowed_vlans = vlan_range_to_list(
                want["trunk"]["trunk_allowed_vlans"],
            )
            vlans_list = [str(num) for num in sorted(allowed_vlans)]
            want["trunk"]["trunk_allowed_vlans"] = ",".join(vlans_list)


def clear_interface(want, have):
    commands = []

    if "mode" in have and want.get("mode") is None:
        commands.append("no switchport mode")

    if "access" in have and not want.get("access"):
        commands.append("no switchport access vlan")

    has_trunk = have.get("trunk") or {}
    wants_trunk = want.get("trunk") or {}
    if "trunk_allowed_vlans" in has_trunk and "trunk_allowed_vlans" not in wants_trunk:
        commands.append("no switchport trunk allowed vlan")
    if "trunk_allowed_vlans" in has_trunk and "trunk_allowed_vlans" in wants_trunk:
        for con in [want, have]:
            expand_trunk_allowed_vlans(con)
        want_allowed_vlans = want["trunk"].get("trunk_allowed_vlans")
        has_allowed_vlans = has_trunk.get("trunk_allowed_vlans")
        allowed_vlans = list(
            set(has_allowed_vlans.split(",")) - set(want_allowed_vlans.split(",")),
        )
        if allowed_vlans:
            allowed_vlans = ",".join(
                ["{0}".format(vlan) for vlan in allowed_vlans],
            )

            commands.append(
                "switchport trunk allowed vlan remove {0}".format(
                    allowed_vlans,
                ),
            )

    if "native_vlan" in has_trunk and "native_vlan" not in wants_trunk:
        commands.append("no switchport trunk native vlan")
    return commands



from __future__ import absolute_import, division, print_function

__metaclass__ = type

import pytest
from django.utils.timezone import now

from awx.main.models.ad_hoc_commands import AdHocCommand


@pytest.mark.django_db
def test_ad_hoc_command_wait_successful(run_module, admin_user):
    command = AdHocCommand.objects.create(status='successful', started=now(), finished=now())
    result = run_module('ad_hoc_command_wait', dict(command_id=command.id), admin_user)
    result.pop('invocation', None)
    result['elapsed'] = float(result['elapsed'])
    assert result.pop('finished', '')[:10] == str(command.finished)[:10]
    assert result.pop('started', '')[:10] == str(command.started)[:10]
    assert result.pop('status', "successful"), result
    assert result.get('changed') is False


@pytest.mark.django_db
def test_ad_hoc_command_wait_failed(run_module, admin_user):
    command = AdHocCommand.objects.create(status='failed', started=now(), finished=now())
    result = run_module('ad_hoc_command_wait', dict(command_id=command.id), admin_user)
    result.pop('invocation', None)
    result['elapsed'] = float(result['elapsed'])
    assert result.pop('finished', '')[:10] == str(command.finished)[:10]
    assert result.pop('started', '')[:10] == str(command.started)[:10]
    assert result.get('changed') is False
    assert result.pop('status', "failed"), result


@pytest.mark.django_db
def test_ad_hoc_command_wait_not_found(run_module, admin_user):
    result = run_module('ad_hoc_command_wait', dict(command_id=42), admin_user)
    result.pop('invocation', None)
    assert result == {"failed": True, "msg": "Unable to wait on ad hoc command 42; that ID does not exist."}



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: iam_policy
version_added: 5.0.0
short_description: Manage inline IAM policies for users, groups, and roles
description:
  - Allows uploading or removing inline IAM policies for IAM users, groups or roles.
  - To administer managed policies please see M(community.aws.iam_user), M(community.aws.iam_role),
    M(amazon.aws.iam_group) and M(community.aws.iam_managed_policy).
  - This module was originally added to C(community.aws) in release 1.0.0.
options:
  iam_type:
    description:
      - Type of IAM resource.
    required: true
    choices: [ "user", "group", "role"]
    type: str
  iam_name:
    description:
      - Name of IAM resource you wish to target for policy actions. In other words, the user name, group name or role name.
    required: true
    type: str
  policy_name:
    description:
      - The name label for the policy to create or remove.
    required: true
    type: str
  policy_json:
    description:
      - A properly json formatted policy as string.
    type: json
  state:
    description:
      - Whether to create or delete the IAM policy.
    choices: [ "present", "absent"]
    default: present
    type: str
  skip_duplicates:
    description:
      - When O(skip_duplicates=true) the module looks for any policies that match the document you pass in.
        If there is a match it will not make a new policy object with the same rules.
    default: false
    type: bool

author:
  - "Jonathan I. Davila (@defionscode)"
  - "Dennis Podkovyrin (@sbj-ss)"
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Advanced example, create two new groups and add a READ-ONLY policy to both
# groups.
- name: Create Two Groups, Mario and Luigi
  amazon.aws.iam_group:
    name: "{{ item }}"
    state: present
  loop:
    - Mario
    - Luigi
  register: new_groups

- name: Apply READ-ONLY policy to new groups that have been recently created
  amazon.aws.iam_policy:
    iam_type: group
    iam_name: "{{ item.iam_group.group.group_name }}"
    policy_name: "READ-ONLY"
    policy_json: "{{ lookup('template', 'readonly.json.j2') }}"
    state: present
  loop: "{{ new_groups.results }}"

# Create a new S3 policy with prefix per user
- name: Create S3 policy from template
  amazon.aws.iam_policy:
    iam_type: user
    iam_name: "{{ item.user }}"
    policy_name: "s3_limited_access_{{ item.prefix }}"
    state: present
    policy_json: "{{ lookup('template', 's3_policy.json.j2') }}"
    loop:
      - user: s3_user
        prefix: s3_user_prefix
"""

RETURN = r"""
user_name:
    description: Name of IAM user.
    returned: When I(iam_type=user)
    type: str
    sample: "ExampleUser001"
group_name:
    description: Name of IAM group.
    returned: When I(iam_type=group)
    type: str
    sample: "ExampleGroup001"
role_name:
    description: Name of IAM role.
    returned: When I(iam_type=role)
    type: str
    sample: "ExampleRole001"
policy_names:
    description: A list of names of the inline policies embedded in the specified IAM resource (user, group, or role).
    returned: always
    type: list
    elements: str
    sample: ["READ-ONLY"]
diff:
    description: A dict representing difference between policies applied on IAM resource (user, group, or role).
    returned: always
    type: dict
    contains:
        before:
            description: The policy that exists on IAM resource before new policy is applied.
            returned: always
            type: dict
            sample: {
                        "READ-ONLY": {
                            "Statement": [
                                {
                                    "Action": "ec2:DescribeAccountAttributes",
                                    "Effect": "Deny",
                                    "Resource": "*",
                                    "Sid": "VisualEditor0"
                                }
                            ],
                            "Version": "2012-10-17"
                        }
                    }
        after:
            description: The current policy on IAM resource after new policy is applied.
            returned: always
            type: dict
            sample: {
                        "READ-ONLY": {
                            "Statement": [
                                {
                                    "Action": "ec2:DescribeAccountAttributes",
                                    "Effect": "Allow",
                                    "Resource": "*",
                                    "Sid": "VisualEditor0"
                                }
                            ],
                            "Version": "2012-10-17"
                        }
                    }
"""

import json

try:
    from botocore.exceptions import BotoCoreError
    from botocore.exceptions import ClientError
except ImportError:
    pass

from ansible.module_utils.six import string_types

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.policy import compare_policies
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry


class PolicyError(Exception):
    pass


class Policy:
    def __init__(self, client, name, policy_name, policy_json, skip_duplicates, state, check_mode):
        self.client = client
        self.name = name
        self.policy_name = policy_name
        self.policy_json = policy_json
        self.skip_duplicates = skip_duplicates
        self.state = state
        self.check_mode = check_mode
        self.changed = False

        self.original_policies = self.get_all_policies().copy()
        self.updated_policies = {}

    @staticmethod
    def _iam_type():
        return ""

    def _list(self, name):
        return {}

    def list(self):
        try:
            return self._list(self.name).get("PolicyNames", [])
        except is_boto3_error_code("AccessDenied"):
            return []

    def _get(self, name, policy_name):
        return "{}"

    def get(self, policy_name):
        try:
            return self._get(self.name, policy_name)["PolicyDocument"]
        except is_boto3_error_code("AccessDenied"):
            return {}

    def _put(self, name, policy_name, policy_doc):
        pass

    def put(self, policy_doc):
        self.changed = True

        if self.check_mode:
            return

        self._put(self.name, self.policy_name, json.dumps(policy_doc, sort_keys=True))

    def _delete(self, name, policy_name):
        pass

    def delete(self):
        self.updated_policies = self.original_policies.copy()

        if self.policy_name not in self.list():
            self.changed = False
            return

        self.changed = True
        self.updated_policies.pop(self.policy_name, None)

        if self.check_mode:
            return

        self._delete(self.name, self.policy_name)

    def get_policy_text(self):
        try:
            if self.policy_json is not None:
                return self.get_policy_from_json()
        except json.JSONDecodeError as e:
            raise PolicyError(f"Failed to decode the policy as valid JSON: {str(e)}")
        return None

    def get_policy_from_json(self):
        if isinstance(self.policy_json, string_types):
            pdoc = json.loads(self.policy_json)
        else:
            pdoc = self.policy_json
        return pdoc

    def get_all_policies(self):
        policies = {}
        for pol in self.list():
            policies[pol] = self.get(pol)
        return policies

    def create(self):
        matching_policies = []
        policy_doc = self.get_policy_text()
        policy_match = False
        for pol in self.list():
            if not compare_policies(self.original_policies[pol], policy_doc):
                matching_policies.append(pol)
                policy_match = True

        self.updated_policies = self.original_policies.copy()

        if self.policy_name in matching_policies:
            return
        if self.skip_duplicates and policy_match:
            return

        self.put(policy_doc)
        self.updated_policies[self.policy_name] = policy_doc

    def run(self):
        if self.state == "present":
            self.create()
        elif self.state == "absent":
            self.delete()
        return {
            "changed": self.changed,
            self._iam_type() + "_name": self.name,
            "policy_names": self.list(),
            "diff": dict(
                before=self.original_policies,
                after=self.updated_policies,
            ),
        }


class UserPolicy(Policy):
    @staticmethod
    def _iam_type():
        return "user"

    def _list(self, name):
        return self.client.list_user_policies(aws_retry=True, UserName=name)

    def _get(self, name, policy_name):
        return self.client.get_user_policy(aws_retry=True, UserName=name, PolicyName=policy_name)

    def _put(self, name, policy_name, policy_doc):
        return self.client.put_user_policy(
            aws_retry=True, UserName=name, PolicyName=policy_name, PolicyDocument=policy_doc
        )

    def _delete(self, name, policy_name):
        return self.client.delete_user_policy(aws_retry=True, UserName=name, PolicyName=policy_name)


class RolePolicy(Policy):
    @staticmethod
    def _iam_type():
        return "role"

    def _list(self, name):
        return self.client.list_role_policies(aws_retry=True, RoleName=name)

    def _get(self, name, policy_name):
        return self.client.get_role_policy(aws_retry=True, RoleName=name, PolicyName=policy_name)

    def _put(self, name, policy_name, policy_doc):
        return self.client.put_role_policy(
            aws_retry=True, RoleName=name, PolicyName=policy_name, PolicyDocument=policy_doc
        )

    def _delete(self, name, policy_name):
        return self.client.delete_role_policy(aws_retry=True, RoleName=name, PolicyName=policy_name)


class GroupPolicy(Policy):
    @staticmethod
    def _iam_type():
        return "group"

    def _list(self, name):
        return self.client.list_group_policies(aws_retry=True, GroupName=name)

    def _get(self, name, policy_name):
        return self.client.get_group_policy(aws_retry=True, GroupName=name, PolicyName=policy_name)

    def _put(self, name, policy_name, policy_doc):
        return self.client.put_group_policy(
            aws_retry=True, GroupName=name, PolicyName=policy_name, PolicyDocument=policy_doc
        )

    def _delete(self, name, policy_name):
        return self.client.delete_group_policy(aws_retry=True, GroupName=name, PolicyName=policy_name)


def main():
    argument_spec = dict(
        iam_type=dict(required=True, choices=["user", "group", "role"]),
        state=dict(default="present", choices=["present", "absent"]),
        iam_name=dict(required=True),
        policy_name=dict(required=True),
        policy_json=dict(type="json", default=None, required=False),
        skip_duplicates=dict(type="bool", default=False, required=False),
    )
    required_if = [
        ("state", "present", ("policy_json",), True),
    ]

    module = AnsibleAWSModule(argument_spec=argument_spec, required_if=required_if, supports_check_mode=True)

    args = dict(
        client=module.client("iam", retry_decorator=AWSRetry.jittered_backoff()),
        name=module.params.get("iam_name"),
        policy_name=module.params.get("policy_name"),
        policy_json=module.params.get("policy_json"),
        skip_duplicates=module.params.get("skip_duplicates"),
        state=module.params.get("state"),
        check_mode=module.check_mode,
    )
    iam_type = module.params.get("iam_type")

    try:
        if iam_type == "user":
            policy = UserPolicy(**args)
        elif iam_type == "role":
            policy = RolePolicy(**args)
        elif iam_type == "group":
            policy = GroupPolicy(**args)

        module.exit_json(**(policy.run()))
    except (BotoCoreError, ClientError) as e:
        module.fail_json_aws(e)
    except PolicyError as e:
        module.fail_json(msg=str(e))


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2023, Anvitha Jain <anvjain@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_interface_policy_spine_policy_group
short_description: Manage spine access interface policy groups (infra:SpAccPortGrp)
description:
- Manage spine access interface policy groups on Cisco ACI fabrics.
options:
  policy_group:
    description:
    - The name of the spine access interface policy group.
    type: str
    aliases: [ name, policy_group_name, spine_policy_group_name ]
  description:
    description:
    - Description of the spine access interface
    type: str
    aliases: [ descr ]
  name_alias:
    description:
    - The alias of the current object. This relates to the nameAlias field in ACI.
    type: str
  link_level_policy:
    description:
    - The name of the link level policy used by the spine access interface
    type: str
    aliases: [ link_level_policy_name ]
  link_flap_policy:
    description:
    - The name of the link flap policy used by the spine access interface
    type: str
    aliases: [ link_flap_policy_name ]
  cdp_policy:
    description:
    - The name of the cdp policy used by the spine access interface
    type: str
    aliases: [ cdp_policy_name ]
  mac_sec_policy:
    description:
    - The name of the mac sec policy used by the spine access interface
    type: str
    aliases: [ mac_sec_policy_name ]
  attached_entity_profile:
    description:
    - The name of the attached entity profile used by the spine access interface
    type: str
    aliases: [ aep_name, aep ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

notes:
seealso:
- name: APIC Management Information Model reference
  description: More information about the internal APIC classes B(infra:SpAccPortGrp).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Anvitha Jain (@anvjain)
"""

EXAMPLES = r"""
- name: Create a Spine Interface Policy Group
  cisco.aci.aci_interface_policy_spine_policy_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    name: spinepolicygroupname
    description: policygroupname description
    link_level_policy: somelinklevelpolicy
    link_flap_policy: somelinkflappolicy
    cdp_policy: somecdppolicy
    mac_sec_policy: somemacsecpolicy
    attached_entity_profile: someattachedentityprofile
    state: present
  delegate_to: localhost

- name: Query all Spine Access Port Policy Groups of type link
  cisco.aci.aci_interface_policy_spine_policy_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result

- name: Query a specific Lead Access Port Policy Group
  cisco.aci.aci_interface_policy_spine_policy_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    name: spinepolicygroupname
    state: query
  delegate_to: localhost
  register: query_result

- name: Delete an Interface policy Spine Policy Group
  cisco.aci.aci_interface_policy_spine_policy_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    name: spinepolicygroupname
    state: absent
  delegate_to: localhost
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        policy_group=dict(type="str", aliases=["policy_group_name", "spine_policy_group_name", "name"]),  # Not required for querying all objects
        description=dict(type="str", aliases=["descr"]),
        name_alias=dict(type="str"),
        link_level_policy=dict(type="str", aliases=["link_level_policy_name"]),
        link_flap_policy=dict(type="str", aliases=["link_flap_policy_name"]),
        cdp_policy=dict(type="str", aliases=["cdp_policy_name"]),
        mac_sec_policy=dict(type="str", aliases=["mac_sec_policy_name"]),
        attached_entity_profile=dict(type="str", aliases=["aep_name", "aep"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["policy_group"]],
            ["state", "present", ["policy_group"]],
        ],
    )

    policy_group = module.params.get("policy_group")
    description = module.params.get("description")
    name_alias = module.params.get("name_alias")
    link_level_policy = module.params.get("link_level_policy")
    link_flap_policy = module.params.get("link_flap_policy")
    cdp_policy = module.params.get("cdp_policy")
    mac_sec_policy = module.params.get("mac_sec_policy")
    attached_entity_profile = module.params.get("attached_entity_profile")
    state = module.params.get("state")

    aci = ACIModule(module)

    class_config = dict(
        name=policy_group,
        descr=description,
        nameAlias=name_alias,
    )

    child_configs = [
        dict(
            infraRsHIfPol=dict(
                attributes=dict(
                    tnFabricHIfPolName=link_level_policy,
                ),
            ),
        ),
        dict(
            infraRsLinkFlapPol=dict(
                attributes=dict(
                    tnFabricLinkFlapPolName=link_flap_policy,
                ),
            ),
        ),
        dict(
            infraRsCdpIfPol=dict(
                attributes=dict(
                    tnCdpIfPolName=cdp_policy,
                ),
            ),
        ),
        dict(
            infraRsMacsecIfPol=dict(
                attributes=dict(
                    tnMacsecIfPolName=mac_sec_policy,
                ),
            ),
        ),
    ]

    if attached_entity_profile is not None:
        child_configs.append(
            dict(
                infraRsAttEntP=dict(
                    attributes=dict(
                        tDn="uni/infra/attentp-{0}".format(attached_entity_profile),
                    ),
                ),
            )
        )

    aci.construct_url(
        root_class=dict(
            aci_class="infraSpAccPortGrp",
            aci_rn="infra/funcprof/spaccportgrp-{0}".format(policy_group),
            module_object=policy_group,
            target_filter={"name": policy_group},
        ),
        child_classes=[
            "infraRsHIfPol",
            "infraRsLinkFlapPol",
            "infraRsCdpIfPol",
            "infraRsMacsecIfPol",
            "infraRsAttEntP",
        ],
    )

    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class="infraSpAccPortGrp",
            class_config=class_config,
            child_configs=child_configs,
        )

        aci.get_diff(aci_class="infraSpAccPortGrp")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sda_site_member_member_v1_info
short_description: Information module for Sda Site Member Member V1
description:
- Get all Sda Site Member Member V1.
- API to get devices that are assigned to a site.
version_added: '6.14.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  id:
    description:
    - Id path parameter. Site Id.
    type: str
  offset:
    description:
    - Offset query parameter. Offset/starting index for pagination.
    type: str
  limit:
    description:
    - Limit query parameter. Number of devices to be listed. Default and max supported value is 500.
    type: str
  memberType:
    description:
    - MemberType query parameter. Member type (This API only supports the 'networkdevice' type).
    type: str
  level:
    description:
    - >
      Level query parameter. Depth of site hierarchy to be considered to list the devices. If the provided value
      is -1, devices for all child sites will be listed.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Sites GetDevicesThatAreAssignedToASiteV1
  description: Complete reference of the GetDevicesThatAreAssignedToASiteV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-devices-that-are-assigned-to-a-site
notes:
  - SDK Method used are
    sites.Sites.get_devices_that_are_assigned_to_a_site_v1,

  - Paths used are
    get /dna/intent/api/v1/site-member/{id}/member,

"""

EXAMPLES = r"""
- name: Get all Sda Site Member Member V1
  cisco.dnac.sda_site_member_member_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    offset: string
    limit: string
    memberType: string
    level: string
    id: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "instanceUuid": "string",
        "instanceId": 0,
        "authEntityId": 0,
        "authEntityClass": 0,
        "instanceTenantId": "string",
        "deployPending": "string",
        "instanceVersion": 0,
        "apManagerInterfaceIp": "string",
        "associatedWlcIp": "string",
        "bootDateTime": "string",
        "collectionInterval": "string",
        "collectionIntervalValue": "string",
        "collectionStatus": "string",
        "description": "string",
        "deviceSupportLevel": "string",
        "dnsResolvedManagementAddress": "string",
        "family": "string",
        "hostname": "string",
        "interfaceCount": "string",
        "inventoryStatusDetail": "string",
        "lastUpdateTime": 0,
        "lastUpdated": "string",
        "lineCardCount": "string",
        "lineCardId": "string",
        "lastDeviceResyncStartTime": "string",
        "macAddress": "string",
        "managedAtleastOnce": true,
        "managementIpAddress": "string",
        "managementState": "string",
        "memorySize": "string",
        "paddedMgmtIpAddress": "string",
        "pendingSyncRequestsCount": "string",
        "platformId": "string",
        "reachabilityFailureReason": "string",
        "reachabilityStatus": "string",
        "reasonsForDeviceResync": "string",
        "reasonsForPendingSyncRequests": "string",
        "role": "string",
        "roleSource": "string",
        "serialNumber": "string",
        "series": "string",
        "snmpContact": "string",
        "snmpLocation": "string",
        "softwareType": "string",
        "softwareVersion": "string",
        "tagCount": "string",
        "type": "string",
        "upTime": "string",
        "uptimeSeconds": 0,
        "vendor": "string",
        "displayName": "string"
      }
    ]
"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


class ModuleDocFragment(object):

    # Standard files documentation fragment
    DOCUMENTATION = r'''
options:
    dnac_host:
        description:
          - The Cisco DNA Center hostname.
        type: str
        required: true
    dnac_port:
        description:
          - The Cisco DNA Center port.
        type: int
        default: 443
    dnac_username:
        description:
          - The Cisco DNA Center username to authenticate.
        type: str
        default: admin
        aliases: [ user ]
    dnac_password:
        description:
          - The Cisco DNA Center password to authenticate.
        type: str
    dnac_verify:
        description:
          - Flag to enable or disable SSL certificate verification.
        type: bool
        default: true
    dnac_version:
        description:
          - Informs the SDK which version of Cisco DNA Center to use.
        type: str
        default: 2.3.7.6
    dnac_debug:
        description:
          - Flag for Cisco DNA Center SDK to enable debugging.
        type: bool
        default: false
    validate_response_schema:
        description:
          - Flag for Cisco DNA Center SDK to enable the validation of request bodies against a JSON schema.
        type: bool
        default: true
notes:
    - "Does not support C(check_mode)"
    - "The plugin runs on the control node and does not use any ansible connection plugins, but instead the embedded connection manager from Cisco DNAC SDK"
    - "The parameters starting with dnac_ are used by the Cisco DNAC Python SDK to establish the connection"
'''



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Red Hat Inc.
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = '''
---
module: rpm_ostree_upgrade
short_description: Manage rpm-ostree upgrade transactions
description:
    - Manage an rpm-ostree upgrade transactions.
version_added: 1.5.0
author:
- Adam Miller (@maxamillion)
requirements:
  - rpm-ostree
options:
    os:
        description:
            - The OSNAME upon which to operate.
        type: str
        default: ""
        required: false
    cache_only:
        description:
          - Perform the transaction using only pre-cached data, do not download.
        type: bool
        default: false
        required: false
    allow_downgrade:
        description:
            - Allow for the upgrade to be a chronologically older tree.
        type: bool
        default: false
        required: false
    peer:
        description:
            - Force peer-to-peer connection instead of using a system message bus.
        type: bool
        default: false
        required: false

'''

EXAMPLES = '''
- name: Upgrade the rpm-ostree image without options, accept all defaults
  ansible.posix.rpm_ostree_upgrade:

- name: Upgrade the rpm-ostree image allowing downgrades
  ansible.posix.rpm_ostree_upgrade:
    allow_downgrade: true
'''

RETURN = '''
msg:
    description: The command standard output
    returned: always
    type: str
    sample: 'No upgrade available.'
'''

import os
import traceback

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils._text import to_native, to_text


def rpm_ostree_transaction(module):
    cmd = []
    cmd.append(module.get_bin_path("rpm-ostree"))
    cmd.append('upgrade')

    if module.params['os']:
        cmd += ['--os', module.params['os']]
    if module.params['cache_only']:
        cmd += ['--cache-only']
    if module.params['allow_downgrade']:
        cmd += ['--allow-downgrade']
    if module.params['peer']:
        cmd += ['--peer']

    module.run_command_environ_update = dict(LANG='C', LC_ALL='C', LC_MESSAGES='C')

    rc, out, err = module.run_command(cmd)

    if rc != 0:
        module.fail_json(rc=rc, msg=err)
    else:
        if to_text("No upgrade available.") in to_text(out):
            module.exit_json(msg=out, changed=False)
        else:
            module.exit_json(msg=out, changed=True)


def main():
    module = AnsibleModule(
        argument_spec=dict(
            os=dict(type='str', default=''),
            cache_only=dict(type='bool', default=False),
            allow_downgrade=dict(type='bool', default=False),
            peer=dict(type='bool', default=False),
        ),
    )

    # Verify that the platform is an rpm-ostree based system
    if not os.path.exists("/run/ostree-booted"):
        module.fail_json(msg="Module rpm_ostree_upgrade is only applicable for rpm-ostree based systems.")

    try:
        rpm_ostree_transaction(module)
    except Exception as e:
        module.fail_json(msg=to_native(e), exception=traceback.format_exc())


if __name__ == '__main__':
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The ios vlans fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type


from copy import deepcopy

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.argspec.vlans.vlans import (
    VlansArgs,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.rm_templates.vlans import (
    VlansTemplate,
)


class VlansFacts(object):
    """The ios vlans fact class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = VlansArgs.argument_spec
        spec = deepcopy(self.argument_spec)
        if subspec:
            if options:
                facts_argument_spec = spec[subspec][options]
            else:
                facts_argument_spec = spec[subspec]
        else:
            facts_argument_spec = spec

        self.generated_spec = utils.generate_dict(facts_argument_spec)

    def get_vlans_data(self, connection):
        """Checks device is L2/L3 and returns
        facts gracefully. Does not fail module.
        """
        check_os_type = connection.get_device_info()
        if check_os_type.get("network_os_type") == "L3":
            return ""
        return connection.get("show vlan")

    def get_vlan_conf_data(self, connection):
        return connection.get("show running-config | section ^vlan configuration .+")

    def populate_vlans_config_facts(self, connection, data=None):
        """Process config for Vlans Configurations

        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf

        :rtype: dictionary
        :returns: facts
        """

        if not data:
            data = self.get_vlan_conf_data(connection)

        # parse native config using the Vlan_configurations template
        vlan_configurations_parser = VlansTemplate(
            lines=data.splitlines(),
            module=self._module,
        )
        return vlan_configurations_parser.parse()

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for vlans
        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf
        :rtype: dictionary
        :returns: facts
        """

        objs = []
        mtu_objs = []
        remote_objs = []
        final_objs = []
        pvlan_objs = []
        conf_data = {}

        if not data:
            data = self.get_vlans_data(connection)

        # deals with vlan configuration config only
        conf_data = self.populate_vlans_config_facts(connection, data)

        # operate on a collection of resource x
        config = data.split("\n")
        # Get individual vlan configs separately
        vlan_info = ""
        temp = ""
        vlan_name = True

        for conf in config:
            if len(list(filter(None, conf.split(" ")))) <= 2 and vlan_name:
                temp = temp + conf
                if len(list(filter(None, temp.split(" ")))) <= 2:
                    continue
            if "VLAN Name" in conf:
                vlan_info = "Name"
            elif "VLAN Type" in conf:
                vlan_info = "Type"
                vlan_name = False
            elif "Remote SPAN" in conf:
                vlan_info = "Remote"
                vlan_name = False
            elif "VLAN AREHops" in conf or "STEHops" in conf:
                vlan_info = "Hops"
                vlan_name = False
            elif "Primary Secondary" in conf:
                vlan_info = "Private"
                vlan_name = False
            if temp:
                conf = temp
                temp = ""
            if conf and " " not in filter(None, conf.split("-")) and not conf.split(" ")[0] == "":
                obj = self.render_config(self.generated_spec, conf, vlan_info)
                if "mtu" in obj:
                    mtu_objs.append(obj)
                elif "remote_span" in obj:
                    remote_objs = obj
                elif "tmp_pvlans" in obj:
                    pvlan_objs.append(obj)
                elif obj:
                    objs.append(obj)

        # Appending MTU value to the retrieved dictionary
        for o, m in zip(objs, mtu_objs):
            o.update(m)
            final_objs.append(o)

        # Appending Remote Span value to related VLAN
        if remote_objs:
            if remote_objs.get("remote_span"):
                for each in remote_objs.get("remote_span"):
                    for every in final_objs:
                        if each == every.get("vlan_id"):
                            every.update({"remote_span": True})
                            break

        # Appending private vlan information to related VLAN
        if pvlan_objs:
            pvlan_final = {}
            if len(pvlan_objs) > 0:
                # Sanitize and structure everything
                for data in pvlan_objs:
                    pvdata = data.get("tmp_pvlans")
                    privlan = pvdata.get("primary")
                    secvlan = pvdata.get("secondary")
                    sectype = pvdata.get("sec_type")

                    # Define the secondary VLAN's type
                    if secvlan and (isinstance(secvlan, int) or secvlan.isnumeric()):
                        secvlan = int(secvlan)
                        pvlan_final[secvlan] = {"private_vlan": {"type": sectype}}

                    # Assemble and merge data for primary private VLANs
                    if privlan and (isinstance(privlan, int) or privlan.isnumeric()):
                        privlan = int(privlan)
                        if privlan not in pvlan_final.keys():
                            pvlan_final[privlan] = {
                                "private_vlan": {"type": "primary", "associated": []},
                            }
                        if secvlan and (isinstance(secvlan, int) or secvlan.isnumeric()):
                            pvlan_final[privlan]["private_vlan"]["associated"].append(
                                int(secvlan),
                            )

                # Associate with the proper VLAN in final_objs
                for vlan_id, data in pvlan_final.items():
                    for every in final_objs:
                        if vlan_id == every.get("vlan_id"):
                            every.update(data)

        facts = {}

        if conf_data:
            for vlan in objs:
                if conf_data.get(vlan.get("vlan_id")):
                    member_data = conf_data.pop(vlan.get("vlan_id"))
                    vlan.update(member_data)

            if conf_data:  # if any vlan configuration data is pending add it to facts
                for vlanid, conf in conf_data.items():
                    objs.append(conf)

        if objs:
            facts["vlans"] = []
            params = utils.validate_config(self.argument_spec, {"config": objs})

            for cfg in params["config"]:
                facts["vlans"].append(utils.remove_empties(cfg))
        ansible_facts["ansible_network_resources"].update(facts)

        return ansible_facts

    def render_config(self, spec, conf, vlan_info):
        """
        Render config as dictionary structure and delete keys
          from spec for null values

        :param spec: The facts tree, generated from the argspec
        :param conf: The configuration
        :rtype: dictionary
        :returns: The generated config
        """
        config = deepcopy(spec)

        if vlan_info == "Name" and "VLAN Name" not in conf:
            conf = list(filter(None, conf.split(" ")))
            config["vlan_id"] = int(conf[0])
            config["name"] = conf[1]
            state_idx = 2
            for i in range(2, len(conf)):  # check for index where state starts
                if conf[i] in ["suspended", "active"]:
                    state_idx = i
                    break
                elif conf[i].split("/")[0] in ["sus", "act"]:
                    state_idx = i
                    break
                config["name"] += " " + conf[i]
            try:
                if len(conf[state_idx].split("/")) > 1:
                    _state = conf[state_idx].split("/")[0]
                    if _state == "sus":
                        config["state"] = "suspend"
                    elif _state == "act":
                        config["state"] = "active"
                    config["shutdown"] = "enabled"
                else:
                    if conf[state_idx] == "suspended":
                        config["state"] = "suspend"
                    elif conf[state_idx] == "active":
                        config["state"] = "active"
                    config["shutdown"] = "disabled"
            except IndexError:
                pass
        elif vlan_info == "Type" and "VLAN Type" not in conf:
            conf = list(filter(None, conf.split(" ")))
            config["mtu"] = int(conf[3])
        elif vlan_info == "Remote":
            if len(conf.split(",")) > 1 or conf.isdigit():
                remote_span_vlan = []
                if len(conf.split(",")) > 1:
                    remote_span_vlan = conf.split(",")
                else:
                    remote_span_vlan.append(conf)
                remote_span = []
                for each in remote_span_vlan:
                    split_sp_list = each.split("-")
                    if len(split_sp_list) > 1:  # break range
                        for r_sp in range(
                            int(split_sp_list[0]),
                            int(split_sp_list[1]) + 1,
                        ):
                            remote_span.append(r_sp)
                    else:
                        remote_span.append(int(each))
                config["remote_span"] = remote_span

        elif vlan_info == "Private" and "Primary Secondary" not in conf:
            conf = list(filter(None, conf.split(" ")))

            pri_idx = 0
            sec_idx = 1
            priv_type_idx = 2

            config["tmp_pvlans"] = {
                "primary": conf[pri_idx],
                "secondary": conf[sec_idx],
                "sec_type": conf[priv_type_idx],
            }

        return utils.remove_empties(config)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: autoscaling_group_info
version_added: 5.0.0
short_description: Gather information about EC2 Auto Scaling Groups (ASGs) in AWS
description:
  - Gather information about EC2 Auto Scaling Groups (ASGs) in AWS.
  - Prior to release 5.0.0 this module was called M(community.aws.ec2_asg_info).
    The usage did not change.
  - This module was originally added to C(community.aws) in release 1.0.0.
author:
  - "Rob White (@wimnat)"
options:
  name:
    description:
      - The prefix or name of the auto scaling group(s) you are searching for.
      - "Note: This is a regular expression match with implicit '^' (beginning of string). Append '$' for a complete name match."
    type: str
    required: false
  tags:
    description:
      - >
        A dictionary/hash of tags in the format { tag1_name: 'tag1_value', tag2_name: 'tag2_value' } to match against the auto scaling
        group(s) you are searching for.
    required: false
    type: dict
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Find all groups
  amazon.aws.autoscaling_group_info:
  register: asgs

- name: Find a group with matching name/prefix
  amazon.aws.autoscaling_group_info:
    name: public-webserver-asg
  register: asgs

- name: Find a group with matching tags
  amazon.aws.autoscaling_group_info:
    tags:
      project: webapp
      env: production
  register: asgs

- name: Find a group with matching name/prefix and tags
  amazon.aws.autoscaling_group_info:
    name: myproject
    tags:
      env: production
  register: asgs

- name: Fail if no groups are found
  amazon.aws.autoscaling_group_info:
    name: public-webserver-asg
  register: asgs
  failed_when: "{{ asgs.results | length == 0 }}"

- name: Fail if more than 1 group is found
  amazon.aws.autoscaling_group_info:
    name: public-webserver-asg
  register: asgs
  failed_when: "{{ asgs.results | length > 1 }}"
"""

RETURN = r"""
results:
    description: A list of Auto Scaling Groups.
    returned: always
    type: list
    elements: dict
    contains:
        auto_scaling_group_arn:
            description: The Amazon Resource Name of the ASG.
            returned: success
            type: str
            sample: "arn:aws:autoscaling:us-west-2:123456789012:autoScalingGroup:10787c52-0bcb-427d-82ba-c8e4b008ed2e:autoScalingGroupName/dev11"
        auto_scaling_group_name:
            description: Name of autoscaling group.
            returned: success
            type: str
            sample: "public-webapp-production-1"
        availability_zones:
            description: List of Availability Zones that are enabled for this ASG.
            returned: success
            type: list
            sample: ["us-west-2a", "us-west-2b", "us-west-2a"]
        created_time:
            description: The date and time this ASG was created, in ISO 8601 format.
            returned: success
            type: str
            sample: "2015-11-25T00:05:36.309Z"
        default_cooldown:
            description: The default cooldown time in seconds.
            returned: success
            type: int
            sample: 300
        desired_capacity:
            description: The number of EC2 instances that should be running in this group.
            returned: success
            type: int
            sample: 3
        enabled_metrics:
            description: The metrics enabled for the group.
            returned: success
            type: list
            elements: dict
            sample: [{
                        "granularity": "1Minute",
                        "metric": "GroupAndWarmPoolTotalCapacity"
                    }]
            contains:
                metric:
                    description: Name of the metric.
                    type: str
                    sample: "GroupAndWarmPoolTotalCapacity"
                granularity:
                    description: The granularity of the metric. The only valid value is 1Minute.
                    type: str
                    sample: "1Minute"
        health_check_grace_period:
            description: Length of time in seconds after a new EC2 instance comes into service that Auto Scaling starts checking its health.
            returned: success
            type: int
            sample: 300
        health_check_type:
            description: The service you want the health status from, one of "EC2" or "ELB".
            returned: success
            type: str
            sample: "ELB"
        instances:
            description: List of EC2 instances associated with ASG and their status.
            returned: success
            type: list
            elements: dict
            sample: [
                {
                    "availability_zone": "us-west-2a",
                    "health_status": "Healthy",
                    "instance_id": "i-es22ad25",
                    "launch_configuration_name": "public-webapp-production-1",
                    "launch_template": {
                            "launch_template_id": "lt-0b19eb00123456789",
                            "launch_template_name": "test-template",
                            "version": "1"
                    },
                    "lifecycle_state": "InService",
                    "protected_from_scale_in": "false"
                }
            ]
            contains:
                availability_zone:
                    description: The Availability Zone of the instance.
                    type: str
                    sample: "ap-southeast-2a"
                health_status:
                    description: The last reported health status of the instance.
                    type: str
                    sample: "Healthy"
                instance_id:
                    description: The ID of the instance.
                    type: str
                    sample: "i-012345678"
                instance_type:
                    description: The instance type size of the running instance.
                    type: str
                    sample: "t2.micro"
                launch_template:
                    description: The EC2 launch template to base instance configuration on.
                    type: dict
                    contains:
                        launch_template_id:
                            description: The ID of the launch template.
                            type: str
                        launch_template_name:
                            description: The name of the launch template.
                            type: str
                        version:
                            description: The specific version of the launch template to use.
                            type: int
                lifecycle_state:
                    description: A description of the current lifecycle state.
                    type: str
                protected_from_scale_in:
                    description: Indicates whether the instance is protected from termination by Amazon EC2 Auto Scaling when scaling in.
                    type: bool
        launch_config_name:
            description: >
                Name of launch configuration associated with the ASG. Same as launch_configuration_name,
                provided for compatibility with M(amazon.aws.autoscaling_group) module.
            returned: success
            type: str
            sample: "public-webapp-production-1"
        launch_configuration_name:
            description: Name of launch configuration associated with the ASG.
            returned: success
            type: str
            sample: "public-webapp-production-1"
        lifecycle_hooks:
            description: List of lifecycle hooks for the ASG.
            returned: success
            type: list
            sample: [
                {
                    "AutoScalingGroupName": "public-webapp-production-1",
                    "DefaultResult": "ABANDON",
                    "GlobalTimeout": 172800,
                    "HeartbeatTimeout": 3600,
                    "LifecycleHookName": "instance-launch",
                    "LifecycleTransition": "autoscaling:EC2_INSTANCE_LAUNCHING"
                },
                {
                    "AutoScalingGroupName": "public-webapp-production-1",
                    "DefaultResult": "ABANDON",
                    "GlobalTimeout": 172800,
                    "HeartbeatTimeout": 3600,
                    "LifecycleHookName": "instance-terminate",
                    "LifecycleTransition": "autoscaling:EC2_INSTANCE_TERMINATING"
                }
            ]
        load_balancer_names:
            description: List of load balancers names attached to the ASG.
            returned: success
            type: list
            sample: ["elb-webapp-prod"]
        max_size:
            description: Maximum size of group.
            returned: success
            type: int
            sample: 3
        min_size:
            description: Minimum size of group.
            returned: success
            type: int
            sample: 1
        new_instances_protected_from_scale_in:
            description: Whether or not new instances a protected from automatic scaling in.
            returned: success
            type: bool
            sample: false
        placement_group:
            description: Placement group into which instances are launched, if any.
            returned: success
            type: str
            sample: None
        service_linked_role_arn:
            description: The ARN of the service-linked role that the Auto Scaling group uses to call other Amazon Web Services on your behalf.
            returned: success
            type: str
            sample: "arn:aws:iam::721234567890:role/aws-service-role/autoscaling.amazonaws.com/AWSServiceRoleForAutoScaling"
        suspended_processes:
            description: The suspended processes associated with the group.
            returned: success
            type: list
            elements: dict
            sample: [{
                        "process_name": "AddToLoadBalancer",
                        "suspension_reason": "User suspended at 2018-08-30T14:11:58Z"
                    }]
            contains:
                process_name:
                    description: The name of the suspended process.
                    type: str
                suspension_reason:
                    description: The reason that the process was suspended.
                    type: str
        status:
            description: The current state of the group when DeleteAutoScalingGroup is in progress.
            returned: success
            type: str
            sample: None
        tags:
            description: List of tags for the ASG, and whether or not each tag propagates to instances at launch.
            returned: success
            type: list
            sample: [
                {
                    "key": "Name",
                    "value": "public-webapp-production-1",
                    "resource_id": "public-webapp-production-1",
                    "resource_type": "auto-scaling-group",
                    "propagate_at_launch": "true"
                },
                {
                    "key": "env",
                    "value": "production",
                    "resource_id": "public-webapp-production-1",
                    "resource_type": "auto-scaling-group",
                    "propagate_at_launch": "true"
                }
            ]
        target_group_arns:
            description: List of ARNs of the target groups that the ASG populates.
            returned: success
            type: list
            sample: [
                "arn:aws:elasticloadbalancing:ap-southeast-2:123456789012:targetgroup/target-group-host-hello/1a2b3c4d5e6f1a2b",
                "arn:aws:elasticloadbalancing:ap-southeast-2:123456789012:targetgroup/target-group-path-world/abcd1234abcd1234"
            ]
        target_group_names:
            description: List of names of the target groups that the ASG populates.
            returned: success
            type: list
            sample: [
                "target-group-host-hello",
                "target-group-path-world"
            ]
        termination_policies:
            description: A list of termination policies for the group.
            returned: success
            type: list
            elements: str
            sample: ["Default"]
        traffic_sources:
            description: The traffic sources associated with this Auto Scaling group.
            returned: success
            type: list
            sample: [{
                        "identifier": "arn:aws:elasticloadbalancing:us-west-2:721234567890:targetgroup/Check-Exter-A4XXXXXXXXXX/8aXXXXXXXXXXXXXX",
                        "type": "elbv2"
                    }]
            contains:
                identifier:
                    description: Identifies the traffic source.
                    type: str
                type:
                    description: Provides additional context for the value of Identifier.
                    type: str
        vpc_zone_identifier:
            description: One or more subnet IDs, if applicable, separated by commas.
            returned: success
            type: str
            sample: "subnet-0352db1247ae60b40,subnet-01f266bb9928ddaee"
"""

import re

try:
    import botocore
except ImportError:
    pass  # caught by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.elb_utils import AnsibleELBv2Error
from ansible_collections.amazon.aws.plugins.module_utils.elb_utils import describe_target_groups
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule


def match_asg_tags(tags_to_match, asg):
    for key, value in tags_to_match.items():
        for tag in asg["Tags"]:
            if key == tag["Key"] and value == tag["Value"]:
                break
        else:
            return False
    return True


def find_asgs(conn, module, name=None, tags=None):
    """
    Args:
        conn (boto3.AutoScaling.Client): Valid Boto3 ASG client.
        name (str): Optional name of the ASG you are looking for.
        tags (dict): Optional dictionary of tags and values to search for.

    Basic Usage:
        >>> name = 'public-webapp-production'
        >>> tags = { 'env': 'production' }
        >>> conn = boto3.client('autoscaling', region_name='us-west-2')
        >>> results = find_asgs(name, conn)

    Returns:
        List
        [
            {
                "auto_scaling_group_arn": (
                    "arn:aws:autoscaling:us-west-2:123456789012:autoScalingGroup:58abc686-9783-4528-b338-3ad6f1cbbbaf:"
                    "autoScalingGroupName/public-webapp-production"
                ),
                "auto_scaling_group_name": "public-webapp-production",
                "availability_zones": ["us-west-2c", "us-west-2b", "us-west-2a"],
                "created_time": "2016-02-02T23:28:42.481000+00:00",
                "default_cooldown": 300,
                "desired_capacity": 2,
                "enabled_metrics": [],
                "health_check_grace_period": 300,
                "health_check_type": "ELB",
                "instances":
                [
                    {
                        "availability_zone": "us-west-2c",
                        "health_status": "Healthy",
                        "instance_id": "i-047a12cb",
                        "launch_configuration_name": "public-webapp-production-1",
                        "lifecycle_state": "InService",
                        "protected_from_scale_in": false
                    },
                    {
                        "availability_zone": "us-west-2a",
                        "health_status": "Healthy",
                        "instance_id": "i-7a29df2c",
                        "launch_configuration_name": "public-webapp-production-1",
                        "lifecycle_state": "InService",
                        "protected_from_scale_in": false
                    }
                ],
                "launch_config_name": "public-webapp-production-1",
                "launch_configuration_name": "public-webapp-production-1",
                "lifecycle_hooks":
                [
                    {
                        "AutoScalingGroupName": "public-webapp-production-1",
                        "DefaultResult": "ABANDON",
                        "GlobalTimeout": 172800,
                        "HeartbeatTimeout": 3600,
                        "LifecycleHookName": "instance-launch",
                        "LifecycleTransition": "autoscaling:EC2_INSTANCE_LAUNCHING"
                    },
                    {
                        "AutoScalingGroupName": "public-webapp-production-1",
                        "DefaultResult": "ABANDON",
                        "GlobalTimeout": 172800,
                        "HeartbeatTimeout": 3600,
                        "LifecycleHookName": "instance-terminate",
                        "LifecycleTransition": "autoscaling:EC2_INSTANCE_TERMINATING"
                    }
                ],
                "load_balancer_names": ["public-webapp-production-lb"],
                "max_size": 4,
                "min_size": 2,
                "new_instances_protected_from_scale_in": false,
                "placement_group": None,
                "status": None,
                "suspended_processes": [],
                "tags":
                [
                    {
                        "key": "Name",
                        "propagate_at_launch": true,
                        "resource_id": "public-webapp-production",
                        "resource_type": "auto-scaling-group",
                        "value": "public-webapp-production"
                    },
                    {
                        "key": "env",
                        "propagate_at_launch": true,
                        "resource_id": "public-webapp-production",
                        "resource_type": "auto-scaling-group",
                        "value": "production"
                    }
                ],
                "target_group_names": [],
                "target_group_arns": [],
                "termination_policies":
                [
                    "Default"
                ],
                "vpc_zone_identifier":
                [
                    "subnet-a1b1c1d1",
                    "subnet-a2b2c2d2",
                    "subnet-a3b3c3d3"
                ]
            }
        ]
    """

    try:
        asgs_paginator = conn.get_paginator("describe_auto_scaling_groups")
        asgs = asgs_paginator.paginate().build_full_result()
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to describe AutoScalingGroups")

    if not asgs:
        return asgs

    try:
        elbv2 = module.client("elbv2")
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError):
        # This is nice to have, not essential
        elbv2 = None
    matched_asgs = []

    if name is not None:
        # if the user didn't specify a name
        name_prog = re.compile(r"^" + name)

    for asg in asgs["AutoScalingGroups"]:
        if name:
            matched_name = name_prog.search(asg["AutoScalingGroupName"])
        else:
            matched_name = True

        if tags:
            matched_tags = match_asg_tags(tags, asg)
        else:
            matched_tags = True

        if matched_name and matched_tags:
            asg = camel_dict_to_snake_dict(asg)
            # compatibility with autoscaling_group module
            if "launch_configuration_name" in asg:
                asg["launch_config_name"] = asg["launch_configuration_name"]
            # workaround for https://github.com/ansible/ansible/pull/25015
            if "target_group_ar_ns" in asg:
                asg["target_group_arns"] = asg["target_group_ar_ns"]
                del asg["target_group_ar_ns"]
            if asg.get("target_group_arns"):
                if elbv2:
                    try:
                        # Limit of 20 similar to https://docs.aws.amazon.com/elasticloadbalancing/latest/APIReference/API_DescribeLoadBalancers.html
                        tg_chunk_size = 20
                        asg["target_group_names"] = []
                        tg_chunks = [
                            asg["target_group_arns"][i: i + tg_chunk_size]
                            for i in range(0, len(asg["target_group_arns"]), tg_chunk_size)
                        ]  # fmt: skip
                        for chunk in tg_chunks:
                            asg["target_group_names"].extend(
                                [tg["TargetGroupName"] for tg in describe_target_groups(elbv2, TargetGroupArns=chunk)]
                            )
                    except AnsibleELBv2Error as e:
                        module.fail_json_aws(e, msg="Failed to describe Target Groups")
            else:
                asg["target_group_names"] = []
            # get asg lifecycle hooks if any
            try:
                asg_lifecyclehooks = conn.describe_lifecycle_hooks(AutoScalingGroupName=asg["auto_scaling_group_name"])
                asg["lifecycle_hooks"] = asg_lifecyclehooks["LifecycleHooks"]
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(e, msg="Failed to fetch information about ASG lifecycle hooks")
            matched_asgs.append(asg)

    return matched_asgs


def main():
    argument_spec = dict(
        name=dict(type="str"),
        tags=dict(type="dict"),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    asg_name = module.params.get("name")
    asg_tags = module.params.get("tags")

    autoscaling = module.client("autoscaling")

    results = find_asgs(autoscaling, module, name=asg_name, tags=asg_tags)
    module.exit_json(results=results)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2020 Cole Neubauer, (@coleneubauer)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = '''
module: azure_rm_aduser_info

version_added: "1.4.0"

short_description: Get Azure Active Directory user info

description:
    - Get Azure Active Directory user info.

options:
    object_id:
        description:
            - The object id for the user.
            - returns the user who has this object ID.
            - Mutually exclusive with I(user_principal_name), I(attribute_name), I(odata_filter) and I(all).
        type: str
    user_principal_name:
        description:
            - The principal name of the user.
            - returns the user who has this principal name.
            - Mutually exclusive with I(object_id), I(attribute_name), I(odata_filter) and I(all).
        type: str
    attribute_name:
        description:
            - The name of an attribute that you want to match to attribute_value.
            - If I(attribute_name) is not a collection type it will return users where I(attribute_name) is equal to I(attribute_value).
            - If I(attribute_name) is a collection type it will return users where I(attribute_value) is in I(attribute_name).
            - Mutually exclusive with I(object_id), I(user_principal_name), I(odata_filter) and I(all).
            - Required together with I(attribute_value).
        type: str
    attribute_value:
        description:
            - The value to match attribute_name to.
            - If I(attribute_name) is not a collection type it will return users where I(attribute_name) is equal to I(attribute_value).
            - If I(attribute_name) is a collection type it will return users where I(attribute_value) is in I(attribute_name).
            - Required together with I(attribute_name).
        type: str
    odata_filter:
        description:
            - Returns users based on the the OData filter passed into this parameter.
            - Mutually exclusive with I(object_id), I(attribute_name), I(user_principal_name) and I(all).
        type: str
    all:
        description:
            - If C(True), will return all users in tenant.
            - If C(False) will return no users.
            - It is recommended that you instead identify a subset of users and use filter.
            - Mutually exclusive with I(object_id), I(attribute_name), I(odata_filter) and I(user_principal_name).
        type: bool
extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Cole Neubauer(@coleneubauer)

'''

EXAMPLES = '''
- name: Using user_principal_name
  azure.azcollection.azure_rm_aduser_info:
    user_principal_name: user@contoso.com

- name: Using object_id
  azure.azcollection.azure_rm_aduser_info:
    object_id: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx

- name: Using attribute mailNickname - not a collection
  azure.azcollection.azure_rm_aduser_info:
    attribute_name: mailNickname
    attribute_value: users_mailNickname

- name: Using attribute proxyAddresses - a collection
  azure.azcollection.azure_rm_aduser_info:
    attribute_name: proxyAddresses
    attribute_value: SMTP:user@contoso.com

- name: Using Filter mailNickname
  azure.azcollection.azure_rm_aduser_info:
    odata_filter: mailNickname eq 'user@contoso.com'

- name: Using Filter proxyAddresses
  azure.azcollection.azure_rm_aduser_info:
    odata_filter: proxyAddresses/any(c:c eq 'SMTP:user@contoso.com')
'''

RETURN = '''
object_id:
    description:
        - The object_id for the user.
    type: str
    returned: always
    sample: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
display_name:
    description:
        - The display name of the user.
    returned: always
    type: str
    sample: John Smith
user_principal_name:
    description:
        - The principal name of the user.
    returned: always
    type: str
    sample: jsmith@contoso.com
mail_nickname:
    description:
        - The mail alias for the user.
    returned: always
    type: str
    sample: jsmith
mail:
    description:
        - The primary email address of the user.
    returned: always
    type: str
    sample: John.Smith@contoso.com
account_enabled:
    description:
        - Whether the account is enabled.
    returned: always
    type: bool
    sample: False
user_type:
    description:
        - A string value that can be used to classify user types in your directory.
    returned: always
    type: str
    sample: Member
mobile_phone:
    description:
        - The mobile phone number of the user.
    type: str
    returned: always
    sample: '+1234567890123'
company_name:
    description:
        - The name of the company that the user is associated with.
    type: str
    returned: always
    sample: "Test Company"
on_premises_extension_attributes:
    description:
        - Contains extensionAttributes1-15 for the user.
        - These extension attributes are also known as Exchange custom attributes 1-15.
        - For an onPremisesSyncEnabled user, the source of authority for this set of properties is the on-premises and is read-only.
        - For a cloud-only user (where onPremisesSyncEnabled is false), these properties can be set during the creation or update of a user object.
        - For a cloud-only user previously synced from on-premises Active Directory, these properties are read-only in Microsoft Graph/
          but can be fully managed through the Exchange Admin Center or the Exchange Online V2 module in PowerShell.
    type: dict
    returned: always
    sample: {}
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBase

try:
    import asyncio
    from msgraph.generated.users.users_request_builder import UsersRequestBuilder
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMADUserInfo(AzureRMModuleBase):
    def __init__(self):

        self.module_arg_spec = dict(
            user_principal_name=dict(type='str'),
            object_id=dict(type='str'),
            attribute_name=dict(type='str'),
            attribute_value=dict(type='str'),
            odata_filter=dict(type='str'),
            all=dict(type='bool'),
        )

        self.user_principal_name = None
        self.object_id = None
        self.attribute_name = None
        self.attribute_value = None
        self.odata_filter = None
        self.all = None
        self.log_path = None
        self.log_mode = None

        self.results = dict(changed=False)

        mutually_exclusive = [['odata_filter', 'attribute_name', 'object_id', 'user_principal_name', 'all']]
        required_together = [['attribute_name', 'attribute_value']]
        required_one_of = [['odata_filter', 'attribute_name', 'object_id', 'user_principal_name', 'all']]

        super(AzureRMADUserInfo, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                supports_check_mode=True,
                                                supports_tags=False,
                                                mutually_exclusive=mutually_exclusive,
                                                required_together=required_together,
                                                required_one_of=required_one_of,
                                                is_ad_resource=True)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        ad_users = []

        try:
            self._client = self.get_msgraph_client()

            if self.user_principal_name is not None:
                ad_users = [asyncio.get_event_loop().run_until_complete(self.get_user(self.user_principal_name))]
            elif self.object_id is not None:
                ad_users = [asyncio.get_event_loop().run_until_complete(self.get_user(self.object_id))]
            elif self.attribute_name is not None and self.attribute_value is not None:
                try:
                    users = asyncio.get_event_loop().run_until_complete(
                        self.get_users_by_filter("{0} eq '{1}'".format(self.attribute_name, self.attribute_value)))
                    ad_users = list(users.value)
                except Exception as e:
                    # the type doesn't get more specific. Could check the error message but no guarantees that message doesn't change in the future
                    # more stable to try again assuming the first error came from the attribute being a list
                    try:
                        users = asyncio.get_event_loop().run_until_complete(self.get_users_by_filter(
                            "{0}/any(c:c eq '{1}')".format(self.attribute_name, self.attribute_value)))
                        ad_users = list(users.value)
                    except Exception as sub_e:
                        raise
            elif self.odata_filter is not None:  # run a filter based on user input to return based on any given attribute/query
                users = asyncio.get_event_loop().run_until_complete(self.get_users_by_filter(self.odata_filter))
                ad_users = list(users.value)
            elif self.all:
                # this returns as a list, since we parse multiple pages
                ad_users = asyncio.get_event_loop().run_until_complete(self.get_users())
            self.results['ad_users'] = [self.to_dict(user) for user in ad_users]

        except Exception as e:
            self.fail("failed to get ad user info {0}".format(str(e)))

        return self.results

    def on_premises_extension_attributes_to_dict(self, on_premises_extension_attributes):
        extension_attributes = {}
        for index in range(1, 16 + 1):
            attribute_name = f'extension_attribute{index}'
            if hasattr(on_premises_extension_attributes, attribute_name):
                attr_value = getattr(on_premises_extension_attributes, attribute_name)
                if attr_value is not None:
                    extension_attributes[attribute_name] = attr_value
        return extension_attributes

    def to_dict(self, object):
        return dict(
            object_id=object.id,
            display_name=object.display_name,
            user_principal_name=object.user_principal_name,
            mail_nickname=object.mail_nickname,
            mail=object.mail,
            account_enabled=object.account_enabled,
            user_type=object.user_type,
            company_name=object.company_name,
            mobile_phone=object.mobile_phone,
            on_premises_extension_attributes=self.on_premises_extension_attributes_to_dict(object.on_premises_extension_attributes)
        )

    async def get_user(self, object):
        request_configuration = UsersRequestBuilder.UsersRequestBuilderGetRequestConfiguration(
            query_parameters=UsersRequestBuilder.UsersRequestBuilderGetQueryParameters(
                select=["accountEnabled", "displayName", "mail", "mailNickname", "id", "userPrincipalName",
                        "userType", "companyName", "mobilePhone", "onPremisesExtensionAttributes"]
            ),
        )
        return await self._client.users.by_user_id(object).get(request_configuration=request_configuration)

    async def get_users(self):
        request_configuration = UsersRequestBuilder.UsersRequestBuilderGetRequestConfiguration(
            query_parameters=UsersRequestBuilder.UsersRequestBuilderGetQueryParameters(
                select=["accountEnabled", "displayName", "mail", "mailNickname", "id", "userPrincipalName",
                        "userType", "companyName", "mobilePhone", "onPremisesExtensionAttributes"]
            ),
        )
        users = []
        # paginated response can be quite large
        response = await self._client.users.get(request_configuration=request_configuration)
        if response:
            users += response.value
        while response is not None and response.odata_next_link is not None:
            response = await self._client.users.with_url(response.odata_next_link).get(request_configuration=request_configuration)
            if response:
                users += response.value

        return users

    async def get_users_by_filter(self, filter):
        return await self._client.users.get(
            request_configuration=UsersRequestBuilder.UsersRequestBuilderGetRequestConfiguration(
                query_parameters=UsersRequestBuilder.UsersRequestBuilderGetQueryParameters(
                    filter=filter,
                    select=["accountEnabled", "displayName", "mail", "mailNickname", "id", "userPrincipalName",
                            "userType", "companyName", "onPremisesExtensionAttributes"],
                    count=True
                ),
            ))


def main():
    AzureRMADUserInfo()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import annotations

import typing

if typing.TYPE_CHECKING:
    from typing import Optional

    from ..transformation import AnsibleAWSResource
    from ..transformation import AnsibleAWSResourceList
    from ..transformation import BotoResource
    from ..transformation import BotoResourceList

from ..transformation import boto3_resource_list_to_ansible_dict
from ..transformation import boto3_resource_to_ansible_dict


def _inject_asg_name(
    instance: BotoResource,
    group_name: Optional[str] = None,
) -> BotoResource:
    if not group_name:
        return instance
    if "AutoScalingGroupName" in instance:
        return instance
    instance["AutoScalingGroupName"] = group_name
    return instance


def normalize_autoscaling_instance(
    instance: BotoResource,
    group_name: Optional[str] = None,
) -> AnsibleAWSResource:
    """Converts an AutoScaling Instance from the CamelCase boto3 format to the snake_case Ansible format.

    Also handles inconsistencies in the output between describe_autoscaling_group() and describe_autoscaling_instances().
    """
    if not instance:
        return instance

    # describe_autoscaling_group doesn't add AutoScalingGroupName
    instance = _inject_asg_name(instance, group_name)

    try:
        # describe_autoscaling_group and describe_autoscaling_instances aren't consistent
        instance["HealthStatus"] = instance["HealthStatus"].upper()
    except KeyError:
        pass

    return boto3_resource_to_ansible_dict(instance, force_tags=False)


def normalize_autoscaling_instances(
    autoscaling_instances: BotoResourceList,
    group_name: Optional[str] = None,
) -> AnsibleAWSResourceList:
    """Converts a list of AutoScaling Instances from the CamelCase boto3 format to the snake_case Ansible format"""
    if not autoscaling_instances:
        return autoscaling_instances
    autoscaling_instances = [normalize_autoscaling_instance(i, group_name) for i in autoscaling_instances]
    return sorted(autoscaling_instances, key=lambda d: d.get("instance_id", None))


def normalize_autoscaling_groups(autoscaling_groups: BotoResourceList) -> AnsibleAWSResourceList:
    """Converts a list of AutoScaling Groups from the CamelCase boto3 format to the snake_case Ansible format"""
    autoscaling_groups = boto3_resource_list_to_ansible_dict(autoscaling_groups)
    return sorted(autoscaling_groups, key=lambda d: d.get("auto_scaling_group_name", None))



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sites_device_credentials_v1
short_description: Resource module for Sites Device Credentials V1
description:
- Manage operation update of the resource Sites Device Credentials V1.
- >
   Updates device credential settings for a site; `null` values indicate that the setting will be inherited from the
   parent site; empty objects `{}` indicate that the credential is unset, and that no credential of that type will be
   used for the site.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  cliCredentialsId:
    description: Sites Device Credentials's cliCredentialsId.
    suboptions:
      credentialsId:
        description: The `id` of the credentials.
        type: str
    type: dict
  httpReadCredentialsId:
    description: Sites Device Credentials's httpReadCredentialsId.
    suboptions:
      credentialsId:
        description: The `id` of the credentials.
        type: str
    type: dict
  httpWriteCredentialsId:
    description: Sites Device Credentials's httpWriteCredentialsId.
    suboptions:
      credentialsId:
        description: The `id` of the credentials.
        type: str
    type: dict
  id:
    description: Id path parameter. Site Id, retrievable from the `id` attribute in
      `/dna/intent/api/v1/sites`.
    type: str
  snmpv2cReadCredentialsId:
    description: Sites Device Credentials's snmpv2cReadCredentialsId.
    suboptions:
      credentialsId:
        description: The `id` of the credentials.
        type: str
    type: dict
  snmpv2cWriteCredentialsId:
    description: Sites Device Credentials's snmpv2cWriteCredentialsId.
    suboptions:
      credentialsId:
        description: The `id` of the credentials.
        type: str
    type: dict
  snmpv3CredentialsId:
    description: Sites Device Credentials's snmpv3CredentialsId.
    suboptions:
      credentialsId:
        description: The `id` of the credentials.
        type: str
    type: dict
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Network Settings UpdateDeviceCredentialSettingsForASiteV1
  description: Complete reference of the UpdateDeviceCredentialSettingsForASiteV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-device-credential-settings-for-a-site
notes:
  - SDK Method used are
    network_settings.NetworkSettings.update_device_credential_settings_for_a_site_v1,

  - Paths used are
    put /dna/intent/api/v1/sites/{id}/deviceCredentials,

"""

EXAMPLES = r"""
- name: Update all
  cisco.dnac.sites_device_credentials_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    cliCredentialsId:
      credentialsId: string
    httpReadCredentialsId:
      credentialsId: string
    httpWriteCredentialsId:
      credentialsId: string
    id: string
    snmpv2cReadCredentialsId:
      credentialsId: string
    snmpv2cWriteCredentialsId:
      credentialsId: string
    snmpv3CredentialsId:
      credentialsId: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": {
        "url": "string",
        "taskId": "string"
      }
    }
"""



#!/usr/bin/python
#
# Copyright (c) 2017 Fred-sun, <xiuxi.sun@qq.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_automationrunbook_info
version_added: '1.12.0'
short_description: Get Azure automation runbook facts
description:
    - Get facts of automation runbook.

options:
    resource_group:
        description:
            - The name of the resource group.
        type: str
        required: True
    automation_account_name:
        description:
            - The name of the automation account.
        type: str
        required: True
    name:
        description:
            - The name of the automation runbook.
        type: str
    show_content:
        description:
            - Wether retrieve the content of runbook identified by runbook name.
        type: bool
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.
        type: list
        elements: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Fred Sun (@Fred-sun)

'''

EXAMPLES = '''
- name: Get details of an automation account
  azure_rm_automationrunbook_info:
    resource_group: "{{ resource_group }}"
    automation_account_name: "{{ account-name }}"
    name: "{{ runbook-name }}"

- name: List automation runbook in the account
  azure_rm_automationrunbook_info:
    resource_group: "{{ resource_group }}"
    automation_account_name: "{{ account-name }}"

- name: Get details of an automation account
  azure_rm_automationrunbook_info:
    resource_group: "{{ resource_group }}"
    automation_account_name: "{{ account-name }}"
    name: "{{ runbook-name }}"
    show_content: true
'''

RETURN = '''
automation_runbook:
    description:
        - List of automation runbook dicts.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID.
            type: str
            returned: always
            sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups
                     /myResourceGroup/providers/Microsoft.Automation/automationAccounts/test/runbooks/runbook_name"
        resource_group:
            description:
                - Resource group name.
            type: str
            returned: always
            sample: myResourceGroup
        name:
            description:
                - Resource name.
            type: str
            returned: always
            sample: Testing
        location:
            description:
                - Resource location.
            type: str
            returned: always
            sample: eastus
        creation_time:
            description:
                - The resource creation date time.
            type: str
            returned: always
            sample: "2022-03-24T06:30:54.116666+00:00"
        job_count:
            description:
                - The job count of the runbook.
            type: int
            returned: always
            sample: 3
        last_modified_by:
            description:
                - The resource last modifier.
            type: str
            returned: always
            sample: Fred-sun
        last_modified_time:
            description:
                - The last person to update the resource.
            type: str
            returned: always
            sample: "2022-03-25T06:30:54.116666+00:00"
        log_activity_trace:
            description:
                - The option to log activity trace of the runbook.
            type: int
            returned: always
            sample: 3
        log_progress:
            description:
                - Whether show progress log option.
            type: bool
            returned: always
            sample: True
        log_verbose:
            description:
                - Whether show verbose log option.
            type: bool
            returned: always
            sample: True
        output_types:
            description:
                - The runbook output type.
            type: list
            returned: always
            sample: []
        runbook_content_link:
            description:
                - The publish runbook content link.
            type: str
            returned: always
            sample: null
        state:
            description:
                - The resource state.
            type: str
            returned: always
            sample: Published
        tags:
            description:
                - The resource tags.
            type: list
            returned: always
            sample: { 'key1': 'value1' }
        type:
            description:
                - The resource automation runbook type.
            type: str
            returned: always
            sample: "Microsoft.Automation/AutomationAccounts/Runbooks"
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    pass


class AzureRMAutomationRunbookInfo(AzureRMModuleBase):
    def __init__(self):
        # define user inputs into argument
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str'
            ),
            automation_account_name=dict(
                type='str',
                required=True
            ),
            show_content=dict(
                type='bool'
            ),
            tags=dict(
                type='list',
                elements='str'
            )
        )
        # store the results of the module operation
        self.results = dict()
        self.resource_group = None
        self.name = None
        self.tags = None
        self.automation_account_name = None
        self.show_content = None

        super(AzureRMAutomationRunbookInfo, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=False, facts_module=True)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec) + ['tags']:
            setattr(self, key, kwargs[key])

        if self.name and self.show_content:
            runbooks = [self.get_content()]
        elif self.name:
            runbooks = [self.get()]
        else:
            runbooks = self.list_by_automaiton_account()
        self.results['automation_runbook'] = [self.to_dict(x) for x in runbooks if x and self.has_tags(x.tags, self.tags)]
        return self.results

    def get_content(self):
        try:
            return self.automation_client.runbook.get(self.resource_group, self.automation_account_name, self.name)
        except ResourceNotFoundError as exc:
            pass

    def get(self):
        try:
            return self.automation_client.runbook.get(self.resource_group, self.automation_account_name, self.name)
        except ResourceNotFoundError as exc:
            pass

    def list_by_automaiton_account(self):
        result = []
        try:
            resp = self.automation_client.runbook.list_by_automation_account(self.resource_group, self.automation_account_name)
            while True:
                result.append(resp.next())
        except StopIteration:
            pass
        except Exception as exc:
            pass
        return result

    def to_dict(self, runbook):
        if not runbook:
            return None
        runbook_dict = dict(
            id=runbook.id,
            type=runbook.type,
            name=runbook.name,
            tags=runbook.tags,
            location=runbook.location,
            runbook_type=runbook.runbook_type,
            runbook_content_link=runbook.publish_content_link,
            state=runbook.state,
            log_verbose=runbook.log_verbose,
            log_progress=runbook.log_progress,
            log_activity_trace=runbook.log_activity_trace,
            job_count=runbook.job_count,
            output_types=runbook.output_types,
            last_modified_by=runbook.last_modified_by,
            last_modified_time=runbook.last_modified_time,
            creation_time=runbook.creation_time,
            description=runbook.description
        )

        return runbook_dict


def main():
    AzureRMAutomationRunbookInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: sda_port_assignment_for_access_point
short_description: Resource module for Sda Port Assignment For Access Point
description:
- This module represents an alias of the module sda_port_assignment_for_access_point_v1
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  authenticateTemplateName:
    description: Authenticate TemplateName associated to Fabric Site.
    type: str
    version_added: 4.0.0
  dataIpAddressPoolName:
    description: Ip Pool Name, that is assigned to INFRA_VN.
    type: str
    version_added: 4.0.0
  deviceManagementIpAddress:
    description: Management Ip Address of the edge device.
    type: str
    version_added: 4.0.0
  interfaceDescription:
    description: Details or note of interface port assignment.
    type: str
    version_added: 4.0.0
  interfaceName:
    description: Interface Name of the edge device.
    type: str
  siteNameHierarchy:
    description: Path of sda Fabric Site.
    type: str
    version_added: 4.0.0
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for SDA AddPortAssignmentForAccessPointInSDAFabricV1
  description: Complete reference of the AddPortAssignmentForAccessPointInSDAFabricV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!add-port-assignment-for-access-point-in-sda-fabric
- name: Cisco DNA Center documentation for SDA DeletePortAssignmentForAccessPointInSDAFabricV1
  description: Complete reference of the DeletePortAssignmentForAccessPointInSDAFabricV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-port-assignment-for-access-point-in-sda-fabric
notes:
  - SDK Method used are
    sda.Sda.add_port_assignment_for_access_point,
    sda.Sda.delete_port_assignment_for_access_point,

  - Paths used are
    post /dna/intent/api/v1/business/sda/hostonboarding/access-point,
    delete /dna/intent/api/v1/business/sda/hostonboarding/access-point,
  - It should be noted that this module is an alias of sda_port_assignment_for_access_point_v1

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.sda_port_assignment_for_access_point:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    authenticateTemplateName: string
    dataIpAddressPoolName: string
    deviceManagementIpAddress: string
    interfaceDescription: string
    interfaceName: string
    siteNameHierarchy: string

- name: Delete all
  cisco.dnac.sda_port_assignment_for_access_point:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    deviceManagementIpAddress: string
    interfaceName: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "status": "string",
      "description": "string",
      "taskId": "string",
      "taskStatusUrl": "string",
      "executionStatusUrl": "string",
      "executionId": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sxp_vpns
short_description: Resource module for SXP VPNs
description:
- Manage operations create and delete of the resource SXP VPNs.
- This API creates a SXP VPN.
- This API deletes a SXP VPN.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  id:
    description: Id path parameter.
    type: str
  sxpVpnName:
    description: SXP VPNs's sxpVpnName.
    type: str
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
notes:
  - SDK Method used are
    sxp_vpns.SxpVpns.create_sxp_vpn,
    sxp_vpns.SxpVpns.delete_sxp_vpn_by_id,

  - Paths used are
    post /ers/config/sxpvpns,
    delete /ers/config/sxpvpns/{id},

"""

EXAMPLES = r"""
- name: Delete by id
  cisco.ise.sxp_vpns:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: absent
    id: string

- name: Create
  cisco.ise.sxp_vpns:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    sxpVpnName: string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "sxpVpnName": "string",
      "link": {
        "rel": "string",
        "href": "string",
        "type": "string"
      }
    }
"""



#!/usr/bin/python
#
# Copyright (c) 2022 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_privateendpointconnection_info

version_added: "1.12.0"

short_description: Get private endpoint connection info

description:
    - Get facts for private endpoint connection info.

options:
    name:
        description:
            - The name of the private end point connection.
        type: str
    service_name:
        description:
            - The name of the private link service.
        type: str
        required: true
    resource_group:
        description:
            - The name of the resource group.
        type: str
        required: true

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - xuzhang3 (@xuzhang3)
    - Fred-sun (@Fred-sun)
'''

EXAMPLES = '''
- name: Get private endpoint connection info by name
  azure_rm_privateendpointconnection_info:
    name: pe-connection-name
    service_name: testserviceName
    resource_group: myRG

- name: Get all private endpoint connection info by service name
  azure_rm_privateendpointconnection_info:
    service_name: testserviceName
    resource_group: myRG
'''

RETURN = '''
endpoint_connection:
    description:
        - List of private endpoint connection info.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID of the private endpoint connection.
            sample: "/subscriptions/xxx/resourceGroups/myRG/providers/Microsoft.Network/privateLinkServices/linkservice/privateEndpointConnections/link.09"
            returned: always
            type: str
        name:
            description:
                - Name of the private endpoint connection.
            returned: always
            type: str
            sample: testlink.09
        link_identifier:
            description:
                - The consumer link id.
            returned: always
            type: str
            sample: 536890208
        PrivateEndpoint:
            description:
                - The resource of private end point.
            type: complex
            returned: always
            contains:
                id:
                    description:
                        - The private endpoint resource ID.
                    type: str
                    returned: always
                    sample: "/subscriptions/xxx-xxx/resourceGroups/myRG/providers/Microsoft.Network/privateEndpoints/testlink02"
        private_link_service_connection_state:
            description:
                - A collection of information about the state of the connection between service consumer and provider.
            type: complex
            returned: always
            contains:
                description:
                    description:
                        - The reason for approval/rejection of the connection.
                    returned: always
                    type: str
                    sample: "Auto Approved"
                status:
                    description:
                        - Indicates whether the connection has been Approved/Rejected/Removed by the owner of the service.
                    returned: always
                    type: str
                    sample: Approved
                actions_required:
                    description:
                        - A message indicating if changes on the service provider require any updates on the consumer.
                    type: str
                    returned: always
                    sample: "This is action_required string"
        provisioning_state:
            description:
                - Provisioning state of the resource.
            returned: always
            type: str
            sample: Succeeded
        etag:
            description:
                - A unique read-only string that changes whenever the resource is updated.
            type: str
            returned: always
            sample: "f7d60f37-ea2b-4091-8546-1327f35468c4"
        type:
            description:
                - The resource type.
            type: str
            returned: always
            sample: Microsoft.Network/privateLinkServices/privateEndpointConnections
'''

try:
    from azure.core.exceptions import ResourceNotFoundError
except Exception:
    # This is handled in azure_rm_common
    pass

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


class AzureRMPrivateEndpointConnectionInfo(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(type="str"),
            service_name=dict(type="str", required=True),
            resource_group=dict(type="str", required=True),
        )

        self.name = None
        self.service_name = None
        self.resource_group = None
        self.results = dict(
            changed=False,
        )

        super(AzureRMPrivateEndpointConnectionInfo, self).__init__(self.module_arg_spec,
                                                                   supports_check_mode=True,
                                                                   supports_tags=False,
                                                                   facts_module=True)

    def exec_module(self, **kwargs):
        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        if self.name is not None:
            self.results["endpoint_connection"] = self.get_item()
        else:
            self.results["endpoint_connection"] = self.list_items()

        return self.results

    def get_item(self):
        self.log("Get properties for {0} in {1}".format(self.name, self.service_name))

        try:
            response = self.network_client.private_link_services.get_private_endpoint_connection(self.resource_group, self.service_name, self.name)
            return [self.connect_to_dict(response)]
        except ResourceNotFoundError:
            self.log("Could not get info for {0} in {1}".format(self.name, self.service_name))

        return []

    def list_items(self):
        result = []
        self.log("List all in {0}".format(self.service_name))
        try:
            response = self.network_client.private_link_services.list_private_endpoint_connections(self.resource_group, self.service_name)
            while True:
                result.append(response.next())
        except StopIteration:
            pass
        except Exception as exc:
            self.fail("Failed to list all items in {0}: {1}".format(self.service_name, str(exc)))
        return [self.connect_to_dict(item) for item in result]

    def connect_to_dict(self, connect_info):
        connect = connect_info.as_dict()
        result = dict(
            id=connect.get("id"),
            name=connect.get('name'),
            type=connect.get('type'),
            etag=connect.get('etag'),
            private_endpoint=dict(),
            private_link_service_connection_state=dict(),
            provisioning_state=connect.get('provisioning_state'),
            link_identifier=connect.get('link_identifier')
        )
        if connect.get('private_endpoint') is not None:
            result['private_endpoint']['id'] = connect.get('private_endpoint')['id']

        if connect.get('private_link_service_connection_state') is not None:
            result['private_link_service_connection_state']['status'] = connect.get('private_link_service_connection_state')['status']
            result['private_link_service_connection_state']['description'] = connect.get('private_link_service_connection_state')['description']
            result['private_link_service_connection_state']['actions_required'] = connect.get('private_link_service_connection_state')['actions_required']
        return result


def main():
    AzureRMPrivateEndpointConnectionInfo()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: wireless_enterprise_ssid_info
short_description: Information module for Wireless Enterprise Ssid Info
description:
- This module represents an alias of the module wireless_enterprise_ssid_v1_info
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  ssidName:
    description:
    - >
      SsidName query parameter. Enter the enterprise SSID name that needs to be retrieved. If not entered, all the
      enterprise SSIDs will be retrieved.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Wireless GetEnterpriseSSIDV1
  description: Complete reference of the GetEnterpriseSSIDV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-enterprise-ssid
notes:
  - SDK Method used are
    wireless.Wireless.get_enterprise_ssid_v1,

  - Paths used are
    get /dna/intent/api/v1/enterprise-ssid,
  - It should be noted that this module is an alias of wireless_enterprise_ssid_v1_info

"""

EXAMPLES = r"""
- name: Get all Wireless Enterprise Ssid Info
  cisco.dnac.wireless_enterprise_ssid_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    ssidName: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "instanceUuid": "string",
        "version": 0,
        "ssidDetails": [
          {
            "name": "string",
            "wlanType": "string",
            "enableFastLane": true,
            "securityLevel": "string",
            "authServer": "string",
            "passphrase": "string",
            "trafficType": "string",
            "enableMACFiltering": true,
            "isEnabled": true,
            "isFabric": true,
            "fastTransition": "string",
            "radioPolicy": "string",
            "enableBroadcastSSID": true,
            "nasOptions": [
              "string"
            ],
            "aaaOverride": true,
            "coverageHoleDetectionEnable": true,
            "protectedManagementFrame": "string",
            "multiPSKSettings": [
              {
                "priority": 0,
                "passphraseType": "string",
                "passphrase": "string"
              }
            ],
            "clientRateLimit": 0,
            "enableSessionTimeOut": true,
            "sessionTimeOut": 0,
            "enableClientExclusion": true,
            "clientExclusionTimeout": 0,
            "enableBasicServiceSetMaxIdle": true,
            "basicServiceSetClientIdleTimeout": 0,
            "enableDirectedMulticastService": true,
            "enableNeighborList": true,
            "mfpClientProtection": "string"
          }
        ],
        "groupUuid": "string",
        "inheritedGroupUuid": "string",
        "inheritedGroupName": "string"
      }
    ]
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: wireless_settings_interfaces_v1
short_description: Resource module for Wireless Settings Interfaces V1
description:
- Manage operations create, update and delete of the resource Wireless Settings Interfaces V1.
- This API allows the user to create an interface.
- This API allows the user to delete an interface by ID.
- This API allows the user to update an interface by ID.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  id:
    description: Id path parameter. Interface ID.
    type: str
  interfaceName:
    description: Interface Name.
    type: str
  vlanId:
    description: VLAN ID range is 1-4094.
    type: int
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Wireless CreateInterfaceV1
  description: Complete reference of the CreateInterfaceV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!create-interface
- name: Cisco DNA Center documentation for Wireless DeleteInterfaceV1
  description: Complete reference of the DeleteInterfaceV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-interface
- name: Cisco DNA Center documentation for Wireless UpdateInterfaceV1
  description: Complete reference of the UpdateInterfaceV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-interface
notes:
  - SDK Method used are
    wireless.Wireless.create_interface_v1,
    wireless.Wireless.delete_interface_v1,
    wireless.Wireless.update_interface_v1,

  - Paths used are
    post /dna/intent/api/v1/wirelessSettings/interfaces,
    delete /dna/intent/api/v1/wirelessSettings/interfaces/{id},
    put /dna/intent/api/v1/wirelessSettings/interfaces/{id},

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.wireless_settings_interfaces_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    interfaceName: string
    vlanId: 0

- name: Delete by id
  cisco.dnac.wireless_settings_interfaces_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    id: string

- name: Update by id
  cisco.dnac.wireless_settings_interfaces_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    id: string
    interfaceName: string
    vlanId: 0

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2023, Gaspard Micol (@gmicol) <gmicol@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: aci_firmware_group
short_description: Manage firmware groups (firmware:FwGrp)
description:
- This module creates a firmware group, so that you can apply firmware policy to nodes.
options:
    group:
        description:
        - Name of the firmware group.
        type: str
    policy:
        description:
        - Name of the firmware policy
        - It is important that you use the same name as the policy created with M(cisco.aci.aci_firmware_policy).
        type: str
        aliases: [ firmwarepol ]
    type_group:
        description:
        - Type of the firmware group.
        - The APIC defaults to C(range) when unset during creation.
        type: str
        choices: [ all, all_in_pod, range ]
    description:
        description:
        - Description of the firmware group.
        type: str
        aliases: [ descr ]
    state:
        description:
        - Use C(present) or C(absent) for adding or removing.
        - Use C(query) for listing an object or multiple objects.
        type: str
        default: present
        choices: [ absent, present, query ]
    name_alias:
        description:
        - The alias for the current object. This relates to the nameAlias field in ACI.
        type: str
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

notes:
- The C(policy) must exist before using this module in your playbook.
- The M(cisco.aci.aci_firmware_policy) module can be used for this.
seealso:
- module: cisco.aci.aci_firmware_policy
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(firmware:FwGrp).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
    - Steven Gerhart (@sgerhart)
    - Gaspard Micol (@gmicol)
"""


EXAMPLES = r"""
- name: Create a firmware group
  cisco.aci.aci_firmware_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    group: fmgroup
    policy: fmpolicy1
    state: present
  delegate_to: localhost

- name: Delete a firmware group
  cisco.aci.aci_firmware_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    group: fmgroup
    state: absent
  delegate_to: localhost

- name: Query all firmware groups
  cisco.aci.aci_firmware_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result

- name: Query a specific firmware group
  cisco.aci.aci_firmware_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    group: fmgroup
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.constants import MATCH_TYPE_GROUP_MAPPING


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        group=dict(type="str"),  # Not required for querying all objects
        policy=dict(type="str", aliases=["firmwarepol"]),  # Not required for querying all objects
        type_group=dict(type="str", choices=list(MATCH_TYPE_GROUP_MAPPING.keys())),
        description=dict(type="str", aliases=["descr"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        name_alias=dict(type="str"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["group"]],
            ["state", "present", ["group", "policy"]],
        ],
    )

    state = module.params.get("state")
    group = module.params.get("group")
    policy = module.params.get("policy")
    type_group = MATCH_TYPE_GROUP_MAPPING.get(module.params.get("type_group"))
    description = module.params.get("description")
    name_alias = module.params.get("name_alias")

    aci = ACIModule(module)
    aci.construct_url(
        root_class=dict(
            aci_class="firmwareFwGrp",
            aci_rn="fabric/fwgrp-{0}".format(group),
            target_filter={"name": group},
            module_object=group,
        ),
        child_classes=["firmwareRsFwgrpp"],
    )

    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class="firmwareFwGrp",
            class_config=dict(
                name=group,
                descr=description,
                type=type_group,
                nameAlias=name_alias,
            ),
            child_configs=[
                dict(
                    firmwareRsFwgrpp=dict(
                        attributes=dict(
                            tnFirmwareFwPName=policy,
                        ),
                    ),
                ),
            ],
        )

        aci.get_diff(aci_class="firmwareFwGrp")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2016, Ansible, inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r'''
---
module: win_command
short_description: Executes a command on a remote Windows node
description:
     - The C(win_command) module takes the command name followed by a list of space-delimited arguments.
     - The given command will be executed on all selected nodes. It will not be
       processed through the shell, so variables like C($env:HOME) and operations
       like C("<"), C(">"), C("|"), and C(";") will not work (use the M(ansible.windows.win_shell)
       module if you need these features).
     - For non-Windows targets, use the M(ansible.builtin.command) module instead.
options:
  _raw_params:
    description:
      - The C(win_command) module takes a free form command to run.
      - This is mutually exclusive with the C(cmd) and C(argv) options.
      - There is no parameter actually named '_raw_params'. See the examples!
    type: str
  cmd:
    description:
    - The command and arguments to run.
    - This is mutually exclusive with the C(_raw_params) and C(argv) options.
    type: str
    version_added: '1.11.0'
  argv:
    description:
    - A list that contains the executable and arguments to run.
    - The module will attempt to quote the arguments specified based on the
      L(Win32 C command-line argument rules,https://docs.microsoft.com/en-us/cpp/c-language/parsing-c-command-line-arguments).
    - Not all applications use the same quoting rules so the escaping may not work, for those scenarios use C(cmd) instead.
    type: list
    elements: str
    version_added: '1.11.0'
  creates:
    description:
      - A path or path filter pattern; when the referenced path exists on the target host, the task will be skipped.
    type: path
  removes:
    description:
      - A path or path filter pattern; when the referenced path B(does not) exist on the target host, the task will be skipped.
    type: path
  chdir:
    description:
      - Set the specified path as the current working directory before executing a command.
    type: path
  stdin:
    description:
    - Set the stdin of the command directly to the specified value.
    type: str
  output_encoding_override:
    description:
    - This option overrides the encoding of stdout/stderr output.
    - You can use this option when you need to run a command which ignore the console's codepage.
    - You should only need to use this option in very rare circumstances.
    - This value can be any valid encoding C(Name) based on the output of C([System.Text.Encoding]::GetEncodings()).
      See U(https://docs.microsoft.com/dotnet/api/system.text.encoding.getencodings).
    type: str
notes:
    - If you want to run a command through a shell (say you are using C(<),
      C(>), C(|), etc), you actually want the M(ansible.windows.win_shell) module instead. The
      M(ansible.windows.win_command) module is much more secure as it's not affected by the user's
      environment.
    - C(creates), C(removes), and C(chdir) can be specified after the command. For instance, if you only want to run a command if a certain file does not
      exist, use this.
    - Do not try to use the older style free form format and the newer style cmd/argv format. See the examples for how both of these formats are defined.
seealso:
- module: ansible.builtin.command
- module: community.windows.psexec
- module: ansible.builtin.raw
- module: community.windows.win_psexec
- module: ansible.windows.win_shell
author:
    - Matt Davis (@nitzmahone)
'''

EXAMPLES = r'''
# Older style using the free-form and args format. The command is on the same
# line as the module and 'args' is used to define the options for win_command.
- name: Save the result of 'whoami' in 'whoami_out'
  ansible.windows.win_command: whoami
  register: whoami_out

- name: Run command that only runs if folder exists and runs from a specific folder
  ansible.windows.win_command: wbadmin -backupTarget:C:\backup\
  args:
    chdir: C:\somedir\
    creates: C:\backup\

- name: Run an executable and send data to the stdin for the executable
  ansible.windows.win_command: powershell.exe -
  args:
    stdin: Write-Host test

# Newer style using module options. The command and other arguments are
# defined as module options and are indended like another other module.
- name: Run the 'whoami' executable with the '/all' argument
  ansible.windows.win_command:
    cmd: whoami.exe /all

- name: Run executable in 'C:\Program Files' with a custom chdir
  ansible.windows.win_command:
    # When using cmd, the arguments need to be quoted manually
    cmd: '"C:\Program Files\My Application\run.exe" "argument 1" -force'
    chdir: C:\Windows\TEMP

- name: Run executable using argv and have win_command escape the spaces as needed
  ansible.windows.win_command:
    # When using argv, each entry is quoted in the module
    argv:
      - C:\Program Files\My Application\run.exe
      - argument 1
      - -force
'''

RETURN = r'''
msg:
    description: changed
    returned: always
    type: bool
    sample: true
start:
    description: The command execution start time
    returned: always
    type: str
    sample: '2016-02-25 09:18:26.429568'
end:
    description: The command execution end time
    returned: always
    type: str
    sample: '2016-02-25 09:18:26.755339'
delta:
    description: The command execution delta time
    returned: always
    type: str
    sample: '0:00:00.325771'
stdout:
    description: The command standard output
    returned: always
    type: str
    sample: 'Clustering node rabbit@slave1 with rabbit@main ...'
stderr:
    description: The command standard error
    returned: always
    type: str
    sample: 'ls: cannot access foo: No such file or directory'
cmd:
    description: The command executed by the task
    returned: always
    type: str
    sample: 'rabbitmqctl join_cluster rabbit@main'
rc:
    description: The command return code (0 means success)
    returned: always
    type: int
    sample: 0
stdout_lines:
    description: The command standard output split in lines
    returned: always
    type: list
    sample: ['Clustering node rabbit@slave1 with rabbit@main ...']
stderr_lines:
    description: The command standard error split in lines
    returned: always
    type: list
    sample: "['ls: cannot access foo: No such file or directory']"
'''



# -*- coding: utf-8 -*-
# Copyright 2022 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Acl_interfaces parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


class Acl_interfacesTemplate(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(Acl_interfacesTemplate, self).__init__(lines=lines, tmplt=self, module=module)

    # fmt: off
    PARSERS = [
        {
            'name': 'interface',
            'getval': re.compile(
                r'''
              ^interface\s
              (?P<name>\S+)$''', re.VERBOSE,
            ),
            'setval': 'interface {{ name }}',
            'result': {
                '{{ name }}': {
                    'name': '{{ name }}',
                    'access_groups': {},
                },
            },
            'shared': True,
        },
        {
            "name": "access_groups",
            "getval": re.compile(
                r"""
                \s+(?P<afi>ip|ipv6)
                (\saccess-group\s(?P<acl_name>\S+))?
                (\straffic-filter\s(?P<acl_name_traffic>\S+))?
                \s(?P<direction>\S+)$
                """,
                re.VERBOSE,
            ),
            "setval": "{{ 'ip access-group' if afi == 'ipv4' else 'ipv6 traffic-filter' }} {{ name|string }} {{ direction }}",
            "result": {
                "{{ name }}": {
                    "access_groups": {
                        "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}": {
                            "afi": "{{ 'ipv4' if afi == 'ip' else 'ipv6' }}",
                            "acls": [
                                {
                                    "name": "{{ acl_name|string if acl_name is defined else acl_name_traffic }}",
                                    "direction": "{{ direction }}",
                                },
                            ],
                        },
                    },
                },
            },
        },
    ]
    # fmt: on



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2024, Gaspard Micol (@gmicol) <gmicol@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: aci_l3out_logical_interface_profile
short_description: Manage Layer 3 Outside (L3Out) logical interface profiles (l3ext:LIfP)
description:
- Manage L3Out interface profiles on Cisco ACI fabrics.
options:
  tenant:
    description:
    - The name of an existing tenant.
    type: str
    aliases: [ tenant_name ]
  l3out:
    description:
    - The name of an existing L3Out.
    type: str
    aliases: [ l3out_name ]
  node_profile:
    description:
    - The name of the node profile.
    type: str
    aliases: [ node_profile_name, logical_node ]
  interface_profile:
    description:
    - The name of the logical interface profile.
    type: str
    aliases: [ name, interface_profile_name, logical_interface ]
  nd_policy:
    description:
    - The name of the neighbor discovery interface policy.
    type: str
  egress_dpp_policy:
    description:
    - The name of the egress data plane policing policy.
    type: str
  ingress_dpp_policy:
    description:
    - The name of the ingress data plane policing policy.
    type: str
  qos_priority:
    description:
    - The QoS priority class ID.
    - The APIC defaults to C(unspecified) when unset during creation.
    type: str
    choices: [ level1, level2, level3, level4, level5, level6, unspecified ]
    aliases: [ priority, prio ]
  qos_custom_policy:
    description:
    - The name of the QoS custom policy.
    type: str
    aliases: [ qos_custom_policy_name ]
  pim_v4_interface_profile:
    description:
    - The PIM IPv4 interface profile.
    type: dict
    suboptions:
      tenant:
        description:
        - The name of the tenant to which the PIM IPv4 interface policy belongs.
        type: str
        aliases: [ tenant_name ]
      pim:
        description:
        - The name of the PIM IPv4 interface policy.
        type: str
        aliases: [ pim_interface_policy, name ]
    aliases: [ pim_v4 ]
  pim_v6_interface_profile:
    description:
    - The PIM IPv6 interface profile.
    type: dict
    suboptions:
      tenant:
        description:
        - The name of the tenant to which the PIM IPv6 interface policy belongs.
        type: str
        aliases: [ tenant_name ]
      pim:
        description:
        - The name of the PIM IPv6 interface policy.
        type: str
        aliases: [ pim_interface_policy, name ]
    aliases: [ pim_v6 ]
  igmp_interface_profile:
    description:
    - The IGMP interface profile.
    type: dict
    suboptions:
      tenant:
        description:
        - The name of the tenant to which the IGMP interface policy belongs.
        type: str
        aliases: [ tenant_name ]
      igmp:
        description:
        - The name of the IGMP interface policy.
        type: str
        aliases: [ igmp_interface_policy, name ]
    aliases: [ igmp ]
  description:
    description:
    - The description for the logical interface profile.
    type: str
    aliases: [ descr ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

notes:
- The I(tenant), I(l3out) and I(node_profile) must exist before using this module in your playbook.
  The M(cisco.aci.aci_tenant), M(cisco.aci.aci_l3out) and M(cisco.aci.aci_l3out_logical_node_profile) can be used for this.
seealso:
- module: cisco.aci.aci_tenant
- module: cisco.aci.aci_l3out
- module: cisco.aci.aci_l3out_logical_node_profile
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(l3ext:LIfP).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Marcel Zehnder (@maercu)
- Gaspard Micol (@gmicol)
"""

EXAMPLES = r"""
- name: Add a new interface profile
  cisco.aci.aci_l3out_logical_interface_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    l3out: my_l3out
    node_profile: my_node_profile
    interface_profile: my_interface_profile
    state: present
  delegate_to: localhost

- name: Query an interface profile
  cisco.aci.aci_l3out_logical_interface_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    l3out: my_l3out
    node_profile: my_node_profile
    interface_profile: my_interface_profile
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all interface profiles
  cisco.aci.aci_l3out_logical_interface_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result

- name: Delete an interface profile
  cisco.aci.aci_l3out_logical_interface_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    l3out: my_l3out
    node_profile: my_node_profile
    interface_profile: my_interface_profile
    state: absent
  delegate_to: localhost
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import (
    ACIModule,
    aci_argument_spec,
    aci_annotation_spec,
    aci_owner_spec,
    pim_interface_profile_spec,
    igmp_interface_profile_spec,
)


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        tenant=dict(type="str", aliases=["tenant_name"]),
        l3out=dict(type="str", aliases=["l3out_name"]),
        node_profile=dict(type="str", aliases=["node_profile_name", "logical_node"]),
        interface_profile=dict(type="str", aliases=["name", "interface_profile_name", "logical_interface"]),
        nd_policy=dict(type="str"),
        egress_dpp_policy=dict(type="str"),
        ingress_dpp_policy=dict(type="str"),
        qos_priority=dict(type="str", choices=["level1", "level2", "level3", "level4", "level5", "level6", "unspecified"], aliases=["priority", "prio"]),
        qos_custom_policy=dict(type="str", aliases=["qos_custom_policy_name"]),
        pim_v4_interface_profile=dict(type="dict", options=pim_interface_profile_spec(), aliases=["pim_v4"]),
        pim_v6_interface_profile=dict(type="dict", options=pim_interface_profile_spec(), aliases=["pim_v6"]),
        igmp_interface_profile=dict(type="dict", options=igmp_interface_profile_spec(), aliases=["igmp"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        description=dict(type="str", aliases=["descr"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["tenant", "l3out", "node_profile", "interface_profile"]],
            ["state", "present", ["tenant", "l3out", "node_profile", "interface_profile"]],
        ],
    )

    tenant = module.params.get("tenant")
    l3out = module.params.get("l3out")
    node_profile = module.params.get("node_profile")
    interface_profile = module.params.get("interface_profile")
    nd_policy = module.params.get("nd_policy")
    egress_dpp_policy = module.params.get("egress_dpp_policy")
    ingress_dpp_policy = module.params.get("ingress_dpp_policy")
    qos_priority = module.params.get("qos_priority")
    qos_custom_policy = module.params.get("qos_custom_policy")
    description = module.params.get("description")
    state = module.params.get("state")

    aci = ACIModule(module)

    extra_child_classes = dict(
        pimIPV6IfP=dict(rs_class="pimRsV6IfPol", attribute_input=module.params.get("pim_v6_interface_profile")),
        pimIfP=dict(rs_class="pimRsIfPol", attribute_input=module.params.get("pim_v4_interface_profile")),
        igmpIfP=dict(rs_class="igmpRsIfPol", attribute_input=module.params.get("igmp_interface_profile")),
    )

    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="l3extOut",
            aci_rn="out-{0}".format(l3out),
            module_object=l3out,
            target_filter={"name": l3out},
        ),
        subclass_2=dict(
            aci_class="l3extLNodeP",
            aci_rn="lnodep-{0}".format(node_profile),
            module_object=node_profile,
            target_filter={"name": node_profile},
        ),
        subclass_3=dict(
            aci_class="l3extLIfP",
            aci_rn="lifp-{0}".format(interface_profile),
            module_object=interface_profile,
            target_filter={"name": interface_profile},
        ),
        child_classes=list(extra_child_classes.keys()) + ["l3extRsEgressQosDppPol", "l3extRsIngressQosDppPol", "l3extRsLIfPCustQosPol", "l3extRsNdIfPol"],
    )

    aci.get_existing()

    if state == "present":
        child_configs = [
            dict(l3extRsNdIfPol=dict(attributes=dict(tnNdIfPolName=nd_policy))),
            dict(l3extRsIngressQosDppPol=dict(attributes=dict(tnQosDppPolName=ingress_dpp_policy))),
            dict(l3extRsEgressQosDppPol=dict(attributes=dict(tnQosDppPolName=egress_dpp_policy))),
            dict(l3extRsLIfPCustQosPol=dict(attributes=dict(tnQosCustomPolName=qos_custom_policy))),
        ]
        for class_name, attribute in extra_child_classes.items():
            attribute_input = attribute.get("attribute_input")
            if attribute_input is not None:
                rs_class = attribute.get("rs_class")
                if all(value is None for value in attribute_input.values()) and isinstance(aci.existing, list) and len(aci.existing) > 0:
                    for child in aci.existing[0].get("l3extLIfP", {}).get("children", {}):
                        if child.get(class_name):
                            child_configs.append(
                                {
                                    class_name: dict(
                                        attributes=dict(status="deleted"),
                                    ),
                                }
                            )
                elif all(value is not None for value in attribute_input.values()):
                    if rs_class in ["pimRsV6IfPol", "pimRsIfPol"]:
                        child_configs.append(
                            {
                                class_name: dict(
                                    attributes={},
                                    children=[
                                        {
                                            rs_class: dict(
                                                attributes=dict(
                                                    tDn="uni/tn-{0}/pimifpol-{1}".format(attribute_input.get("tenant"), attribute_input.get("pim"))
                                                )
                                            )
                                        },
                                    ],
                                )
                            }
                        )
                    elif rs_class == "igmpRsIfPol":
                        child_configs.append(
                            {
                                class_name: dict(
                                    attributes={},
                                    children=[
                                        {
                                            rs_class: dict(
                                                attributes=dict(
                                                    tDn="uni/tn-{0}/igmpIfPol-{1}".format(attribute_input.get("tenant"), attribute_input.get("igmp"))
                                                )
                                            )
                                        },
                                    ],
                                )
                            }
                        )

        aci.payload(
            aci_class="l3extLIfP",
            class_config=dict(
                name=interface_profile,
                prio=qos_priority,
                descr=description,
            ),
            child_configs=child_configs,
        )

        aci.get_diff(aci_class="l3extLIfP")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()
